import os
import csv
import sys
import shutil
from glob import glob
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def del_attributes(entry, features):
    attributes = {}
    for key, value in entry.attributes.items():
        if (key not in features):
            attributes[key] = value
    entry.attributes = attributes


def read_gffs(gff_files, feature):
    gffs = {}
    if feature == "transcript":
        gffs["transcript"] = []
        gff_f = open(gff_files, "r")
        for entry in Gff3Parser().entries(gff_f):
            gffs["transcript"].append(entry)
        gff_f.close()
        gffs["transcript"] = sorted(
                gffs["transcript"], key=lambda x: (x.seq_id, x.start,
                                                   x.end, x.strand))
    else:
        num = 0
        for files in gff_files:
            for gff_file in glob(files):
                gffs[num] = []
                gff_f = open(gff_file, "r")
                for entry in Gff3Parser().entries(gff_f):
                    parent = None
                    if (entry.feature == "CDS") or (
                            entry.feature == "exon") or (
                            entry.feature == "repeat_unit") or (
                            entry.feature == "tRNA") or (
                            entry.feature == "rRNA") or (
                            entry.feature == "ncRNA"):
                        if "Parent" in entry.attributes.keys():
                            parent = entry.attributes["Parent"]
                    del_attributes(entry, ["associated_tran", "parent_tran",
                                           "Parent", "Parent"])
                    if parent is not None:
                        entry.attributes["Parent"] = parent
                    entry.attributes["print"] = False
                    gffs[num].append(entry)
                gff_f.close()
                gffs[num] = sorted(
                    gffs[num], key=lambda x: (x.seq_id, x.start,
                                              x.end, x.strand))
                num += 1
    return gffs


def assign_parent(other, tran):
    '''assign the parent transcript to all features'''
    if "Parent" not in other.attributes.keys():
        other.attributes["Parent"] = tran.attributes["ID"]
    else:
        other.attributes["Parent"] = ",".join([
            other.attributes["Parent"], tran.attributes["ID"]])


def compare_tran(tran_gffs, other_gffs, fuzzy_tss, fuzzy_term):
    for tran in tran_gffs["transcript"]:
        for num, others in other_gffs.items():
            for other in others:
                if (tran.seq_id == other.seq_id) and (
                        tran.strand == other.strand) and (
                        other.feature != "source") and (
                        other.feature != "region"):
                    if other.feature == "TSS":
                        if (other.start >= tran.start - fuzzy_tss) and (
                                other.end <= tran.end + fuzzy_tss):
                            assign_parent(other, tran)
                    elif other.feature.lower() == "terminator":
                        if tran.strand == "+":
                            if ((tran.end >= other.start) and (
                                     tran.end <= other.end)) or (
                                    (tran.end <= other.start) and (
                                     (other.start - tran.end) <= fuzzy_term)) or (
                                    (tran.end >= other.end) and (
                                     (tran.end - other.end) <= fuzzy_term)) or (
                                    (tran.start <= other.start) and (
                                     tran.end >= other.end)):
                                assign_parent(other, tran)
                        else:
                            if ((tran.start >= other.start) and (
                                     tran.start <= other.end)) or (
                                    (tran.start <= other.start) and (
                                     (other.start - tran.start) <= fuzzy_term)) or (
                                    (tran.start >= other.end) and (
                                     (tran.start - other.end) <= fuzzy_term)) or (
                                    (tran.start <= other.start) and (
                                     tran.end >= other.end)):
                                assign_parent(other, tran)
                    else:
                        if ((tran.start <= other.start) and (
                                tran.end >= other.end)) or (
                                (tran.start >= other.start) and (
                                tran.end <= other.end)) or (
                                (tran.start <= other.start) and (
                                tran.end >= other.start) and (
                                tran.end <= other.end)) or (
                                (tran.start >= other.start) and (
                                tran.start <= other.end) and (
                                tran.end >= other.end)):
                            assign_parent(other, tran)


def combine_gffs(tran_gffs, other_gffs):
    gffs = []
    o_gffs = []
    s_gffs = []
    if tran_gffs is not None:
        for tran in tran_gffs["transcript"]:
            gffs.append(tran)
            for num, others in other_gffs.items():
                for other in others:
                    if tran.seq_id == other.seq_id:
                        if "Parent" in other.attributes.keys():
                            attributes = {}
                            for key, value in other.attributes.items():
                                if key != "print":
                                    attributes[key] = value
                            if (tran.attributes["ID"] in 
                                    other.attributes["Parent"].split(",")):
                                other.attribute_string = ";".join(
                                    ["=".join(items) for items in 
                                        attributes.items()])
                                other.attributes["print"] = True
                                gffs.append(other)
    for num, others in other_gffs.items():
        for other in others:
            if (other.feature == "source") or (
                    other.feature == "region"):
                s_gffs.append(other)
            if not other.attributes["print"]:
                attributes = {}
                for key, value in other.attributes.items():
                    if key != "print":
                        attributes[key] = value
                other.attribute_string = ";".join(
                    ["=".join(items) for items in attributes.items()])
                o_gffs.append(other)
    return gffs, o_gffs, s_gffs


def print_gff(gffs, o_gffs, s_gffs, output):
    sort_others = sorted(o_gffs, key=lambda x: (x.seq_id, x.start,
                                                x.end, x.strand))
    out = open(output, "w")
    if len(gffs) != 0:
        pre_strain = None
        for gff in gffs:
            if (pre_strain is not None) and (pre_strain != gff.seq_id):
                for other in sort_others:
                    if other.seq_id == pre_strain:
                        if (not other.attributes["print"]):
                            out.write("\t".join([other.info_without_attributes,
                                                 other.attribute_string]) + "\n")
                            other.attributes["print"] = True
            for source in s_gffs:
                if (source.seq_id == gff.seq_id) and (
                        not source.attributes["print"]):
                    out.write("\t".join([source.info_without_attributes,
                                         source.attribute_string]) + "\n")
                    source.attributes["print"] = True
            out.write("\t".join([gff.info_without_attributes,
                                 gff.attribute_string]) + "\n")
            pre_strain = gff.seq_id
        for other in sort_others:
            if other.seq_id == gff.seq_id:
                if (not other.attributes["print"]):
                    out.write("\t".join([other.info_without_attributes,
                                         other.attribute_string]) + "\n")
                    other.attributes["print"] = True
    else:
        for other in sort_others:
            out.write("\t".join([gff.info_without_attributes,
                                 gff.attribute_string]) + "\n")
    out.close()


def run_merge(out_folder, tran, others, fuzzy_term, fuzzy_tss, strain):
    '''merge all features to be one gff file'''
    output = "_".join([strain, "merge_features.gff"])
    if tran is None and others is None:
        print("Error: There is no input file...")
        sys.exit()
    elif (tran is not None) and (others is None):
        shutil.copy(tran, os.path.join(out_folder, output))
    elif others is not None:
        if (tran is not None):
            tran_gffs = read_gffs(tran, "transcript")
            other_gffs = read_gffs(others, "others")
            compare_tran(tran_gffs, other_gffs, fuzzy_tss, fuzzy_term)
        else:
            other_gffs = read_gffs(others, "others")
        gffs, o_gffs, s_gffs = combine_gffs(tran_gffs, other_gffs)
        print_gff(gffs, o_gffs, s_gffs, output)



import os
import csv
import shutil
from subprocess import call
from annogesiclib.seq_editer import SeqEditer


def wget(input_folder, ftp, files_type):
    os.system(" ".join(["wget", "-cP", input_folder, ftp + "/*" + files_type]))


def deal_detect(input_file, file_path, change, input_folder):
    '''deal with the header of fasta file and 
    put the files to corresponding folders'''
    if change:
        shutil.move(input_file, file_path)
        change = False
    SeqEditer().modify_header(file_path)
    with open(os.path.join(file_path)) as fh:
        for line in fh:
            line = line.strip()
            if line.startswith(">"):
                seq_name = line[1:]
    shutil.move(file_path,
                os.path.join(input_folder, seq_name + ".fa"))
    return change, seq_name


def get_file(ftp, input_folder, files_type, target):
    checks = {"detect": False, "change": None}
    filename = None
    files = []
    wget(input_folder, ftp, files_type)
    for file_ in os.listdir(input_folder):
        input_file = os.path.join(input_folder, file_)
        if (file_[-3:] == "fna"):
            filename = file_[0:-3] + "fa"
            checks = {"detect": True, "change": True}
        elif (file_[-5:] == "fasta"):
            filename = file_[0:-5] + "fa"
            checks = {"detect": True, "change": True}
        elif (file_[-2:] == "fa"):
            filename = file_[0:-2] + "fa"
            checks = {"detect": True, "change": True}
        elif (file_[-6:] == "fna.gz") and ("_genomic" in file_):
            if ("_cds_from_genomic" in file_) or (
                    "_rna_from_genomic" in file_):
                os.remove(input_file)
            else:
                filename = file_[0:-6] + "fa"
                checks = {"detect": True, "change": True}
                call(["gunzip", input_file])
                input_file = input_file[:-3]
        elif (file_[-6:] == "gff.gz") or (file_[-3:] == "gff"):
            if ("_genomic" in file_) and (file_[-6:] == "gff.gz"):
                call(["gunzip", input_file])
                input_file = input_file[:-3]
            fh = open(input_file, "r")
            for row in csv.reader(fh, delimiter='\t'):
                if not row[0].startswith("#"):
                    gff_name = row[0]
                    break
            shutil.move(input_file, os.path.join(input_folder,
                                               gff_name + ".gff"))
            fh.close()
        elif (file_[-3:] == "gbk") or (file_[-7:] == "gbff.gz") or (
                file_[-4:] == "gbff"):
            if (file_[-7:] == "gbff.gz") and ("_genomic" in file_):
                call(["gunzip", input_file])
                input_file = input_file[:-3]
            with open(input_file, "r") as g_f:
                for line in g_f:
                    if line[0:7] == "VERSION":
                        data = line[12:].split(" ")
                        break
            shutil.move(input_file, os.path.join(input_folder, data[0] + ".gbk"))
        if checks["detect"]:
            checks["detect"] = False
            checks["change"], seq_name = deal_detect(
                    input_file, filename, checks["change"], input_folder)



from annogesiclib.gff3 import Gff3Parser


def line_to_dict(hit, strain, e_value):
    return {"hit": hit, "strain": strain, "e_value": e_value}


def get_proteins(datas, checks, blast_f):
    '''filter and import the protein hit of blast'''
    proteins = []
    nums = {"index": 0, "hypo": 0}
    for data in datas:
        if (nums["index"] % 4 == 0) and (nums["index"] != 0):
            if "[" in data:
                data1 = data.split("[")
                data2 = data1[1].split("]")
                strain = data2[0].strip()
            else:
                data1 = data.split("Length")
                strain = "NA"
            name = data1[0].strip()
            tag = datas[nums["index"] - 1]
            if ("hypothetical" in name) or (
                    "Hypothetical" in name) or (
                    "unknown" in name) or (
                    "Unknown" in name) or (
                    "predicted coding region" in name) or (
                    "Predicted coding region" in name) or (
                    "PREDICTED:" in name) or (
                    "putative" in name) or ("Putative" in name):
                nums["hypo"] += 1
            if not checks["detect"]:
                for line in blast_f:
                    line = line.strip()
                    if ("Expect" in line) and ("Score" in line) and (
                            "Method" in line):
                        e_value = line.split(",")[1].split(" ")[-1]
                        checks["detect"] = True
                        checks["print"] = True
                        proteins.append({"name": name, "strain": strain,
                                         "e": e_value, "tag": tag})
                        break
            else:
                proteins.append({"name": name, "strain": strain,
                                 "e": e_value, "tag": tag})
        nums["index"] += 1
    return proteins, nums


def detect_hypo(proteins, blasts, type_):
    '''remove the hit which is hypothetical protein or unknown'''
    protein_names = {}
    for protein in proteins:
        name = protein["name"].replace("\n", "")
        if ("hypothetical" not in name) and (
                "Hypothetical" not in name) and (
                "Unknown" not in name) and (
                "unknown" not in name) and (
                "Predicted coding region" not in name) and (
                "predicted coding region" not in name) and (
                "PREDICTED:" not in name) and ("putative" not in name) and (
                "Putative" not in name):
            if (name not in protein_names.keys()):
                protein_names[name] = []
            protein_names[name].append(protein["tag"])
            if type_ != "equal":
                blasts["blast"] = True
    return protein_names, protein["e"]


def detect_nr(line, blast_f, out_t, blasts, prefix):
    '''detect the hit in nr database'''
    checks = {"print": False, "detect": False}
    if line.startswith(">"):
        info = line.replace(">", "")
        for line in blast_f:
            line = line.strip()
            if len(line) != 0:
                info = " ".join([info, line])
            else:
                break
        datas = info.split("|")
        proteins, nums = get_proteins(datas, checks, blast_f)
        if checks["print"]:
            if blasts["hit_num"] < 3:
                protein_names, e = detect_hypo(proteins, blasts, "low")
                if len(protein_names) != 0:
                    for key, value in protein_names.items():
                        out_t.write("{0}\t{1}\t{2}\t{3}\n".format(
                                    prefix, key, ",".join(value), e))
                    blasts["hit_num"] += 1


def detect_srna(line, blast_f, out_t, blasts, prefix):
    '''detect hit in sRNA database'''
    print_ = False
    blasts["name"] = ""
    if line[0] == ">":
        name_complete = False
        blasts["name"] = line[1:]
        blasts["hit_num"] += 1
        for line in blast_f:
            line.strip()
            if line.startswith("Length="):
                name_complete = True
            if not name_complete:
                blasts["name"] = " ".join([blasts["name"], line])
            if "Expect =" in line:
                e_value = line.split(" ")[-1].strip()
                print_ = True
                break
    if print_:
        blasts["name"] = blasts["name"].lstrip().replace("\n", "")
        out_t.write("{0}\t{1}\t{2}\n".format(
                    prefix, blasts["name"], e_value))
        blasts["blast"] = True


def read_gff(srna_file, data_type):
    srnas = []
    srna_f = open(srna_file, "r")
    for entry in Gff3Parser().entries(srna_f):
        attributes = {}
        for key, value in entry.attributes.items():
            if (data_type == "sRNA") and (
               "sRNA_hit" not in key):
                attributes[key] = value
            elif (data_type == "nr") and (
                  "nr_hit" not in key):
                attributes[key] = value
        entry.attributes = attributes
        attribute_string = ";".join(
            ["=".join(items) for items in entry.attributes.items()])
        entry.info = "\t".join([entry.info_without_attributes,
                                attribute_string])
        srnas.append(entry)
    srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return srnas


def print_file(database, out_f, info, srna_hit, nr_hit):
    if database == "sRNA":
        out_f.write("{0};sRNA_hit={1}\n".format(info, srna_hit))
    elif database == "nr":
        out_f.write("{0};nr_hit={1}\n".format(info, nr_hit))


def gen_out_flie(blasts, out_t, prefix, out_f, database, srna, names):
    if not blasts["blast"]:
        out_t.write("{0}\tNA\n".format(prefix))
        print_file(database, out_f,
                   srna.info, "NA", "NA")
    else:
        print_file(database, out_f, srna.info,
                   len(names), blasts["hit_num"])


def get_whole_query(line, blast_f):
    whole_line = line
    if (not line.endswith("+")) and (
            not line.endswith("-")):
        for line in blast_f:
            line = line.strip()
            whole_line = whole_line + line
            if (line.endswith("+")) or (
                    line.endswith("-")):
                return whole_line
    else:
        return whole_line


def extract_blast(blast_result, srna_file, output_file,
                  output_table, database):
    '''extract the result of blast'''
    out_f = open(output_file, "w")
    out_t = open(output_table, "w")
    out_f.write("##gff-version 3\n")
    srnas = read_gff(srna_file, database)
    for srna in srnas:
        blasts = {"hit_num": 0, "blast": False, "name": ""}
        names = []
        prefix = "\t".join([srna.seq_id, srna.attributes["ID"],
                           srna.strand, str(srna.start), str(srna.end)])
        with open(blast_result, "r") as blast_f:
            for line in blast_f:
                line = line.strip()
                if line.startswith("Query= "):
                    line = get_whole_query(line, blast_f)
                    go_out = False
                    query = line.split("=")[1].strip()
                    if (query == ("|".join([
                            srna.attributes["ID"], srna.seq_id,
                            str(srna.start), str(srna.end), srna.strand]))):
                        for line in blast_f:
                            line = line.strip()
                            if line.find("No hits found") != -1:
                                print_file(database, out_f,
                                           srna.info, "NA", "NA")
                                out_t.write("{0}\tNA\n".format(prefix))
                                break
                            elif line.find("Sequences producing "
                                           "significant alignments:") != -1:
                                for line in blast_f:
                                    line = line.strip()
                                    if len(line) != 0:
                                        if line.startswith(
                                                "Effective search space"):
                                            go_out = True
                                            break
                                        if database == "sRNA":
                                            detect_srna(line, blast_f, out_t,
                                                        blasts, prefix)
                                            if (len(blasts["name"]) > 0):
                                                if blasts["name"] not in names:
                                                    names.append(
                                                        blasts["name"])
                                        elif database == "nr":
                                            detect_nr(line, blast_f, out_t,
                                                      blasts, prefix)
                                gen_out_flie(blasts, out_t, prefix, out_f,
                                             database, srna, names)
                                blasts["hit_num"] = 0
                                break
                        if go_out:
                            break
    out_f.close()
    out_t.close()


def extract_energy(srna_file, sec_file, out_file):
    '''extract the folding energy of sRNA'''
    s_f = open(srna_file, "r")
    check = False
    get_length = False
    out = open(out_file, "w")
    out.write("##gff-version 3\n")
    for srna in Gff3Parser().entries(s_f):
        with open(sec_file, "r") as d_f:
            for structure in d_f:
                structure = structure.rstrip('\n')
                if get_length:
                    length = len(structure)
                    get_length = False
                if (structure.startswith(">")):
                    if (("|".join([srna.attributes["ID"], srna.seq_id,
                         str(srna.start),
                         str(srna.end), srna.strand])) == structure[1:]) or (
                        ("|".join([srna.feature, srna.seq_id,
                         str(srna.start),
                         str(srna.end), srna.strand])) == structure[1:]):
                        check = True
                        get_length = True
                if (check) and (
                    (structure[0] == "(") or (
                     structure[0] == ")") or (
                     structure[0] == ".")) and (
                     structure[-1] == ")"):
                    check = False
                    data = structure.split(" ")
                    if (data[-1].find("(") == -1):
                        energy = float(data[-1][0:-1])
                    else:
                        energy = float(data[-1][1:-1])
                    out.write("{0};2d_energy={1:.4f}\n".format(
                              srna.info, (energy / float(length))))
                    break
    s_f.close()
    d_f.close()
    out.close()



class WigParser(object):
    '''parser the wiggle file based on 
    strain, track, position and coverage'''

    def parser(self, wig_fh, strand):
        track = ""
        strain = ""
        for line in wig_fh.readlines():
            line = line.strip()
            datas = line.split(" ")
            if (datas[0] == "variableStep"):
                strain = datas[1].split("=")
                strain = strain[1]
                pre_pos = 0
                first = True
            if (datas[0] == "track"):
                track = datas[2].split("=")
                track = track[1].replace("\"", "")
                pre_pos = 0
                first = True
            if (datas[0] != "track") and (datas[0] != "variableStep"):
                if len(datas) != 2:
                    datas = line.split("\t")
                if int(datas[0]) - 1 != pre_pos:
                    for pos in range(pre_pos + 1, int(datas[0])):
                        yield assign_value(pos, 0, strand, strain, track)
                    pre_pos = int(datas[0])
                    first = True
                if (int(datas[0]) - 1 == pre_pos) or (first):
                    pre_pos = int(datas[0])
                    first = False
                    yield assign_value(datas[0], datas[1],
                                       strand, strain, track)


class assign_value(object):

    def __init__(self, pos, coverage, strand, strain, track):
        self.pos = int(pos)
        if strand == "+":
            self.coverage = float(coverage)
        else:
            self.coverage = -1 * float(coverage)
        self.strand = strand
        self.strain = strain
        self.track = track

    def __str__(self):
        return "{0} {1} {2} {3} {4}".format(
                self.pos, self.coverage, self.strand, self.strain, self.track)



import math
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.helper import Helper


def import_to_operon(start, end, strand):
    return {"start": start, "end": end, "strand": strand}


def get_gene_info(cds):
    if "locus_tag" in cds.attributes.keys():
        feature = cds.attributes["locus_tag"]
    else:
        strand = Helper().get_strand_name(cds.strand)
        feature = "".join([cds.feature, ":", str(cds.start),
                           "-", str(cds.end), "_", strand])
    return feature


def get_term_feature(ta, data, term_fuzzy, features, datas,
                     ta_check_point, data_check_start, data_check_end):
    '''verify and get proper terminator to operon'''
    jump = False
    if (ta.strand == data.strand) and (
            ta.seq_id == data.seq_id) and (
            (math.fabs(data.start - ta_check_point) <= term_fuzzy) or (
            math.fabs(data.end - ta_check_point) <= term_fuzzy) or (
            (ta_check_point >= data.start) and (
            ta_check_point <= data.end))):
        features["detect"] = True
    if (ta.strand == data.strand) and (
            ta.seq_id == data.seq_id):
        if (ta.start <= data_check_start) and (
                ta.end >= data_check_end):
            features["num"] += 1
            datas.append(data)
        elif (ta_check_point >= data.start) and (
                ta_check_point <= data.end):
            features["num"] += 1
            datas.append(data)
    if (ta.seq_id == data.seq_id) and (
            data.start - term_fuzzy > ta.end):
        jump = True
    return jump


def get_tss_feature(ta, data, features, tss_fuzzy, datas, ta_check_point,
                    data_check_start, data_check_end):
    '''verify and get the proper TSS for operon'''
    jump = False
    if (ta.strand == data.strand) and (
            ta.seq_id == data.seq_id) and (
            math.fabs(ta_check_point - data.start) <= tss_fuzzy):
        features["detect"] = True
    if (ta.strand == data.strand) and (
            ta.seq_id == data.seq_id) and (
            ta.start <= data_check_start) and (
            ta.end >= data_check_end):
        features["num"] += 1
        datas.append(data)
    if (ta.seq_id == data.seq_id) and (
            data_check_end > ta.end):
        jump = True
    return jump


def detect_features(ta, inputs, feature, term_fuzzy, tss_fuzzy):
    '''Detect the feature which should group as a operon'''
    features = {"num": 0, "detect": False}
    datas = []
    for data in inputs:
        if (feature == "term"):
            if ta.strand == "+":
                jump_term = get_term_feature(ta, data, term_fuzzy, features,
                                             datas, ta.end, data.start,
                                             data.start - term_fuzzy)
            elif ta.strand == "-":
                jump_term = get_term_feature(ta, data, term_fuzzy, features,
                                             datas, ta.start,
                                             data.end + term_fuzzy, data.end)
            if jump_term:
                break
        elif (feature == "tss"):
            if ta.strand == "+":
                jump_tss = get_tss_feature(ta, data, features, tss_fuzzy,
                                           datas, ta.start,
                                           data.start + tss_fuzzy, data.end)
            elif ta.strand == "-":
                jump_tss = get_tss_feature(ta, data, features, tss_fuzzy,
                                           datas, ta.end, data.end,
                                           data.start - tss_fuzzy)
            if jump_tss:
                break
        else:
            if feature == "gene":
                if (ta.strand == data.strand) and (
                        ta.seq_id == data.seq_id) and (
                        data.feature == "gene"):
                    if ((ta.start <= data.start) and (
                             ta.end >= data.end)) or (
                            (ta.start >= data.start) and (
                             ta.end <= data.end)) or (
                            (ta.start >= data.start) and (
                             ta.start < data.end) and (
                             ta.end > data.end)) or (
                            (ta.start < data.start) and (
                             ta.end <= data.end) and (
                             ta.end > data.start)):
                        features["num"] += 1
                        features["detect"] = True
                        datas.append(data)
    return {"data_list": datas, "num_feature": features["num"],
            "with_feature": features["detect"]}


def check_conflict(genes, pos, strand):
    '''check TSS which is not primary or secondary TSS'''
    conflict = False
    for gene in genes["data_list"]:
        if (gene.strand == strand):
            if (gene.start < pos) and (
                    gene.end >= pos):
                conflict = True
                break
    return conflict


def check_gene(tsss, genes, strand, ta_pos, first, min_length, end,
               operons, operon_pos):
    '''Check TSS and annotated feature. It can group the feature and TSS to 
    be operon or sub-operon'''
    no_count_tsss = []
    for tss in tsss:
        if tss not in no_count_tsss:
            end_points = [ta_pos]
            for pos in tsss:
                if (pos not in no_count_tsss) and (
                        tss.start != pos.start):
                    end_points.append(pos.start)
            end_points.append(end)
            if tss.strand == "+":
                end_points.sort()
            else:
                end_points.sort(reverse=True)
            for point in end_points:
                detect_pos = False
                if tss.strand == "+":
                    for gene in genes["data_list"]:
                        if (gene.seq_id == tss.seq_id) and (
                                gene.strand == tss.strand):
                            if (gene.start >= tss.start) and (
                                    gene.end <= point):
                                detect_pos = True
                                break
                else:
                    for gene in genes["data_list"]:
                        if (gene.seq_id == tss.seq_id) and (
                                gene.strand == tss.strand):
                            if (gene.start >= point) and (
                                    gene.end <= tss.start):
                                detect_pos = True
                                break
                if not detect_pos:
                    no_count_tsss.append(tss)
                else:
                    operon_pos, first = compute_sub_operon(
                        strand, point, ta_pos, first, min_length,
                        end, operons, operon_pos)
                    break


def sub_operon_gene_conflict(tsss, strand, genes, ta_pos, first, min_length,
                             end, operons, operon_pos):
    '''remove the TSS which is not primary or secondary TSS of gene
    This TSS can not form sub-operon'''
    new_tsss = []
    for tss in tsss["data_list"]:
        conflict = check_conflict(genes, tss.start, strand)
        if not conflict:
            new_tsss.append(tss)
    check_gene(new_tsss, genes, strand, ta_pos, first,
               min_length, end, operons, operon_pos)


def sub_operon(strand, tsss, ta_pos, end, genes, min_length):
    '''verify the sub-operon'''
    first = True
    operons = []
    operon_pos = ta_pos
    if tsss["with_feature"]:
        if tsss["num_feature"] == 1:
            pass
        else:
            sub_operon_gene_conflict(
                tsss, strand, genes, ta_pos, first, min_length,
                end, operons, operon_pos)
    else:
        sub_operon_gene_conflict(
            tsss, strand, genes, ta_pos, first,
            min_length, end, operons, operon_pos)
    return operons


def compute_sub_operon(strand, point, ta_pos, first,
                       min_length, end, operons, operon_pos):
    '''For computting and import the sub-operon'''
    if first:
        operon_pos = ta_pos
        first = False
    if math.fabs(point - operon_pos) > min_length:
        if strand == "+":
            operons.append(import_to_operon(operon_pos,
                           point - 1, strand))
            operon_pos = point
        else:
            operons.append(import_to_operon(point + 1,
                           operon_pos, strand))
            operon_pos = point
    return operon_pos, first


def read_gff(ta_file, gff_file, tss_file, terminator_file):
    tas = []
    gffs = []
    tss_gffs = []
    term_gffs = []
    gff_parser = Gff3Parser()
    for ta in gff_parser.entries(open(ta_file)):
        tas.append(ta)
    for entry in gff_parser.entries(open(gff_file)):
        gffs.append(entry)
    for entry in gff_parser.entries(open(tss_file)):
        tss_gffs.append(entry)
    if terminator_file is not False:
        for entry in gff_parser.entries(open(terminator_file)):
            term_gffs.append(entry)
        term_gffs = sorted(term_gffs, key=lambda k: (k.seq_id, k.start,
                                                     k.end, k.strand))
    tas = sorted(tas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    gffs = sorted(gffs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    tss_gffs = sorted(tss_gffs, key=lambda k: (k.seq_id, k.start,
                                               k.end, k.strand))
    return tas, gffs, tss_gffs, term_gffs


def print_file(ta, operons, out, operon_id, whole_operon, tsss,
               terms, genes, whole_gene):
    if len(operons) <= 1:
        out.write("\t".join([operon_id, ta.seq_id,
                  "-".join([str(whole_operon.start), str(whole_operon.end)]),
                  whole_operon.strand, "0", "NA",
                  str(tsss["with_feature"]), str(tsss["num_feature"]),
                  str(terms["with_feature"]), str(terms["num_feature"]), "NA",
                  str(genes["num_feature"]), "NA",
                  ", ".join(whole_gene)]) + "\n")
    else:
        for sub in operons:
            sub_gene = []
            num_sub_gene = 0
            for gene in genes["data_list"]:
                if (sub["strand"] == gene.strand) and (
                        sub["start"] <= gene.start) and (
                        sub["end"] >= gene.end):
                    if "locus_tag" in gene.attributes.keys():
                        sub_gene.append(gene.attributes["locus_tag"])
                    else:
                        sub_gene.append("".join([gene.feature, ":",
                                        str(gene.start), "-", str(gene.end),
                                        "_", gene.strand]))
                    num_sub_gene += 1
            if num_sub_gene == 0:
                sub_gene.append("NA")
            out.write("\t".join([operon_id, ta.seq_id,
                      "-".join([str(whole_operon.start),
                                str(whole_operon.end)]),
                      whole_operon.strand, str(len(operons)),
                      "-".join([str(sub["start"]), str(sub["end"])]),
                      str(tsss["with_feature"]), str(tsss["num_feature"]),
                      str(terms["with_feature"]), str(terms["num_feature"]),
                      str(num_sub_gene), str(genes["num_feature"]),
                      ", ".join(sub_gene), ", ".join(whole_gene)]) + "\n")


def operon(ta_file, tss_file, gff_file, terminator_file, tss_fuzzy,
           term_fuzzy, min_length, out_file):
    '''main part for detection of operon'''
    out = open(out_file, "w")
    out.write("Operon_ID\tstrain\tOperon_position\tstrand\t")
    out.write("Number_of_suboperon\tPosition_of_suboperon\tStart_with_TSS\t")
    out.write("Number_of_TSS\tTerminated_with_terminator\t")
    out.write("Number_of_terminator\tNumber_of_gene_associated_suboperon\t")
    out.write("Number_of_gene_associated_operon\t")
    out.write("Associated_genes_with_suboperon\t")
    out.write("Associated_genes_with_whole_operon\n")
    num_operon = 0
    tas, gffs, tss_gffs, term_gffs = read_gff(ta_file, gff_file, tss_file,
                                              terminator_file)
    for ta in tas:
        whole_gene = []
        check_operon = False
        if (math.fabs(ta.start - ta.end) >= min_length):
            whole_operon = ta
            check_operon = True
            num_operon += 1
        genes = detect_features(ta, gffs, "gene", term_fuzzy, tss_fuzzy)
        tsss = detect_features(ta, tss_gffs, "tss", term_fuzzy, tss_fuzzy)
        if terminator_file is None:
            terms = {"with_feature": "NA", "num_feature": "NA"}
        else:
            terms = detect_features(ta, term_gffs, "term",
                                    term_fuzzy, tss_fuzzy)
        if ta.strand == "+":
            operons = sub_operon(ta.strand, tsss, ta.start,
                                 ta.end, genes, min_length)
        else:
            operons = sub_operon(ta.strand, tsss, ta.end,
                                 ta.start, genes, min_length)
        operon_id = "Operon" + str(num_operon)
        if genes["num_feature"] != 0:
            for gene in genes["data_list"]:
                whole_gene.append(get_gene_info(gene))
        else:
            whole_gene.append("NA")
        if check_operon:
            print_file(ta, operons, out, operon_id, whole_operon,
                       tsss, terms, genes, whole_gene)
    out.close()



import itertools
import numpy as np
from annogesiclib.gff3 import Gff3Parser
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def plot(pri, sec, anti, inter, orph, total, total_more, name,
         feature_name, file_type):
    tsss = [pri, sec, anti, inter, orph]
    ind = np.arange(5)
    width = 0.5
    fig, ax = plt.subplots()
    if feature_name == "processing site":
        plt.text(0.85, 0.95, "Total processing sites", ha='center',
                 va='center', transform=ax.transAxes)
        plt.text(0.85, 0.9, str(total), ha='center', va='center',
                 transform=ax.transAxes)
    elif feature_name == "TSS":
        plt.text(0.9, 0.95, "Total TSSs", ha='center', va='center',
                 transform=ax.transAxes)
        plt.text(0.9, 0.9, str(total), ha='center', va='center',
                 transform=ax.transAxes)
    rects = ax.bar(ind, tsss, width, color='#9999FF')
    ax.set_ylabel("the number of " + feature_name)
    ax.set_xticks(ind + width/2)
    ax.set_xticklabels(('Primary', 'Secondary', 'Antisense',
                        'Internal', 'Orphan'))
    ax.set_xlabel("The type of " + feature_name)
    for rect in rects:
        height = rect.get_height()
        plt.text(rect.get_x()+rect.get_width()/2., 1.05*height,
                 '%d' % int(height), ha='center', va='bottom')
    plt.savefig(file_type + "_class_" + name + ".png")


def stat(tsss, strain, feature_name, out_stat, file_type, out_lib):
    tss_type = {"Primary": [], "Secondary": [], "Internal": [],
                "Antisense": [], "Orphan": []}
    tss_libs = {}
    num_tss = 0
    num_tss_more = 0
    for entry in tsss:
        num_tss += 1
        if entry.attributes["type"].find("Primary") != -1:
            tss_type["Primary"].append(num_tss)
        if entry.attributes["type"].find("Secondary") != -1:
            tss_type["Secondary"].append(num_tss)
        if entry.attributes["type"].find("Antisense") != -1:
            tss_type["Antisense"].append(num_tss)
        if entry.attributes["type"].find("Internal") != -1:
            tss_type["Internal"].append(num_tss)
        if entry.attributes["type"].find("Orphan") != -1:
            tss_type["Orphan"].append(num_tss)
        if "libs" in entry.attributes.keys():
            libs = entry.attributes["libs"].split("&")
            for lib in libs:
                if lib not in tss_libs.keys():
                    tss_libs[lib] = 1
                else:
                    tss_libs[lib] += 1
    for key in tss_type.keys():
        num_tss_more = num_tss_more + len(tss_type[key])
    plot(len(tss_type["Primary"]), len(tss_type["Secondary"]),
         len(tss_type["Antisense"]), len(tss_type["Internal"]),
         len(tss_type["Orphan"]), num_tss, num_tss_more,
         strain, feature_name, file_type)
    out_stat.write(strain + ":\n")
    out_lib.write(strain + ":\n")
    out_lib.write("total TSS are {0}\n".format(num_tss))
    for tss_lib, lib_num in tss_libs.items():
        out_lib.write(": ".join([tss_lib, str(lib_num)]))
        out_lib.write(" ({0})\n".format(lib_num / num_tss))
    out_stat.write("total number of {0} (if one {1} belong to two class, "
                   "it count two times) = {2}\n".format(
                       feature_name, feature_name, num_tss_more))
    out_stat.write("total number of unique {0} (if one {1} belong to "
                   "two class, it count only one time) = {2}\n".format(
                       feature_name, feature_name, num_tss))
    for it in range(1, 5):
        for tss in itertools.combinations(tss_type.keys(), it):
            union = []
            for key in tss:
                union = list(set(tss_type[key]) | set(union))
            out_stat.write("{0} = {1} ({2})\n".format(
                           '-'.join(tss), len(union),
                           float(len(union)) / float(num_tss)))
    out_stat.write("\n")


def stat_tsspredator(tss_file, file_type, stat_file, lib_file):
    if file_type == "processing":
        feature_name = "processing site"
    else:
        feature_name = "TSS"
    tsss = []
    tsss_strain = {}
    pre_seq_id = ""
    out_stat = open(stat_file, "w")
    out_lib = open(lib_file, "w")
    gff_parser = Gff3Parser()
    fh = open(tss_file)
    for entry in gff_parser.entries(fh):
        if entry.seq_id != pre_seq_id:
            pre_seq_id = entry.seq_id
            tsss_strain[entry.seq_id] = []
        tsss_strain[entry.seq_id].append(entry)
        tsss.append(entry)
    tsss = sorted(tsss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    if len(tsss_strain) > 1:
        stat(tsss, "All_strains", feature_name, out_stat, file_type, out_lib)
    for strain in tsss_strain.keys():
        stat(tsss_strain[strain], strain, feature_name,
             out_stat, file_type, out_lib)
    out_stat.close()
    out_lib.close()
    fh.close()



import csv
import copy
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


def plot_bar(cutoffs, strain, out_snp, type_):
    name = []
    for index in range(0, len(cutoffs) + 1):
        name.append(index * 10)
    ind = np.arange(len(cutoffs))
    width = 0.5
    plt.figure(figsize=(20, 15))
    plt.bar(ind, cutoffs, width, color='#FF9999')
    plt.ylabel('the number of SNPs', fontsize=20)
    plt.xlabel('QUAL of SNP in transcripts', fontsize=20)
    plt.xlim([0, len(cutoffs) + 1])
    plt.xticks(ind+width-0.75, name, fontsize=18, rotation=40)
    plt.yticks(fontsize=18)
    plt.savefig("_".join([out_snp, strain, "SNP_QUAL", type_]))
    plt.clf()


def row_in_list(row):
    info = ""
    filt = "."
    snps = {"strain": None, "pos": -1, "id": "", "ref": "",
            "alt": "", "qual": -1, "filter": "", "info": "",
            "depth": -1, "all_info": "", "indel": -1, "frac": -1,
            "dp4_sum": -1, "dp4_frac": -1}
    if len(row) >= 8:
        snps = {"strain": row[0], "pos": int(row[1]), "id": row[2],
                "ref": row[3], "alt": row[4], "qual": float(row[5]),
                "filter": filt, "info": "", "depth": -1,
                "all_info": "\t".join(row), "indel": -1, "frac": -1}
        infos = row[7].split(";")
        snps["info"] = infos
        for info in infos:
            datas = info.split("=")
            if len(datas) > 1:
                if datas[0] == "DP":
                    snps["depth"] = int(datas[1])
                if datas[0] == "IDV":
                    snps["indel"] = int(datas[1])
                if datas[0] == "IMF":
                    snps["frac"] = float(datas[1])
                if datas[0] == "DP4":
                    dp4s = datas[1].split(",")
                    snps["dp4_sum"] = int(dp4s[2]) + int(dp4s[3])
                    total = (int(dp4s[0]) + int(dp4s[1]) + 
                             int(dp4s[2]) + int(dp4s[3]))
                    snps["dp4_frac"] = float(snps["dp4_sum"] / total)
        return snps
    else:
        return snps


def gen_ref(snps, pos, refs, num):
    if num == 1:
        for snp in snps:
            refs.append(":".join([str(pos), snp["alt"]]))
    else:
        new_refs = []
        for snp in snps:
            for ref in refs:
                new_refs.append(ref + "_" + str(pos) + ":" + snp["alt"])
        refs = copy.deepcopy(new_refs)
    return refs


def change(snp, seq):
    start_point = snp["pos"] - 1 + seq["num_mod"]
    end_point = snp["pos"] - 1 + len(snp["ref"]) + seq["num_mod"]
    if len(snp["ref"]) == len(snp["alt"]):
        if seq["seq"][start_point: end_point].upper() == snp["ref"].upper():
            seq["seq"] = (seq["seq"][:start_point] +
                          snp["alt"].lower() + seq["seq"][end_point:])
    if len(snp["ref"]) > len(snp["alt"]):
        if seq["seq"][start_point: end_point].upper() == snp["ref"].upper():
            seq["seq"] = (seq["seq"][:start_point] +
                          snp["alt"].lower() + seq["seq"][end_point:])
            seq["num_mod"] = seq["num_mod"] - (
                    len(snp["ref"]) - len(snp["alt"]))
    if len(snp["ref"]) < len(snp["alt"]):
        if seq["seq"][start_point: end_point].upper() == snp["ref"].upper():
            seq["seq"] = (seq["seq"][:start_point] +
                          snp["alt"].lower() + seq["seq"][end_point:])
            seq["num_mod"] = seq["num_mod"] - (
                    len(snp["ref"]) - len(snp["alt"]))


def get_n_a_value(para, depth_file, min_sample):
    '''get the corresponding number of cutoff'''
    tag = para.split("_")[0]
    value = float(para.split("_")[-1])
    if tag == "a":
        fh = open(depth_file, "r")
        total = 0
        num = 0
        for row in csv.reader(fh, delimiter='\t'):
            total = total + int(row[-1])
            num += 1
        avg_dep = float(total / num)
        fh.close()
        value = avg_dep * value
    elif tag == "n":
        value = (min_sample * value)
    elif tag == "r":
        pass
    return value

def apply_filter(filters, snp, snps):
    '''apply the filter to remove the false positives'''
    exclude = False
    for filt in filters:
        tag = filt.split("_")[0]
        cutoff = filt.split("_")[-1]
        for info in snp["info"]:
            if tag in info:
                value = float(info.split("=")[-1])
                if cutoff[0] == "b":
                    if value < float(cutoff[1:]):
                        exclude = True
                if cutoff[0] == "s":
                    if value > float(cutoff[1:]):
                        exclude = True
    if not exclude:
        snps.append(snp)

def import_data(snp_file, args_snp, bam_number, depth_file):
    snps = []
    raw_snps = []
    dess = []
    max_quals = {}
    max_quals["All_strain"] = 0
    pre_strain = ""
    cutoff_sum = get_n_a_value(args_snp.dp4_sum, depth_file,
                               args_snp.min_sample)
    cutoff_frac = float(args_snp.dp4_frac)
    depth_s = get_n_a_value(args_snp.depth_s, depth_file,
                            args_snp.min_sample)
    depth_b = get_n_a_value(args_snp.depth_b, depth_file,
                            args_snp.min_sample)
    idv = get_n_a_value(args_snp.idv, depth_file,
                        args_snp.min_sample)
    imf = float(args_snp.imf)
    fh = open(snp_file, "r")
    for row in csv.reader(fh, delimiter="\t"):
        if row[0].startswith("##"):
            dess.append(row[0])
        if row[0].startswith("#"):
            continue
        else:
            snp = row_in_list(row)
            raw_snps.append(snp)
            if snp["strain"]:
                if snp["strain"] != pre_strain:
                    pre_strain = snp["strain"]
                    max_quals[snp["strain"]] = 0
                if snp["qual"] > max_quals[snp["strain"]]:
                    max_quals[snp["strain"]] = snp["qual"]
                if snp["qual"] > max_quals["All_strain"]:
                    max_quals["All_strain"] = snp["qual"]
                if (snp["depth"] >= depth_s) and (
                        snp["depth"] <= depth_b) and (
                        snp["dp4_sum"] >= cutoff_sum) and (
                        snp["dp4_frac"] >= cutoff_frac):
                    if snp["indel"] == -1:
                        apply_filter(args_snp.filters, snp, snps)
                    else:
                        if (snp["frac"] >= imf) and (
                                snp["indel"] >= idv):
                            apply_filter(args_snp.filters, snp, snps)
    fh.close()
    return max_quals, snps, dess, raw_snps


def check_overlap(new_snps, overlaps):
    count = 0
    element = 0
    first_overlap = True
    printeds = []
    count_overlap = len(overlaps)
    for key, value in new_snps.items():
        if first_overlap:
            for overlap in overlaps:
                if "print" in overlap.keys():
                    element += 1
                    printeds.append(overlap)
                else:
                    break
            first_overlap = False
        if ("print" not in overlaps[element].keys()):
            if len(printeds) == 0:
                value.append(overlaps[element])
                count += 1
            else:
                for printed in printeds:
                    if printed not in value:
                        value.append(overlaps[element])
                        count += 1
        if count_overlap != 0:
            if count == (len(new_snps.keys()) / count_overlap):
                overlaps[element]["print"] = True
                element += 1
                if element >= len(overlaps):
                    break
                count = 0


def overlap_position(qual_snps):
    '''deal with the conflict position of SNPs'''
    first = True
    qual_nooverlap_snps = {}
    num_overlap = 1
    qual_nooverlap_snps[num_overlap] = []
    conflicts = []
    for snp1 in qual_snps:
        overlaps = []
        overlaps.append(snp1)
        for snp2 in qual_snps:
            if (snp1 != snp2) and (snp1["strain"] == snp2["strain"]) and (
                    (snp2["pos"] - snp1["pos"] < len(snp1["ref"]))) and (
                    "print" not in snp2.keys()):
                overlaps.append(snp2)
        if len(overlaps) != 1:
            conflicts.append(overlaps)
            if first:
                for overlap in overlaps:
                    qual_nooverlap_snps[num_overlap] = []
                    qual_nooverlap_snps[num_overlap].append(overlap)
                    num_overlap += 1
                    overlap["print"] = True
                num_overlap = 1
                first = False
            else:
                new_snps = qual_nooverlap_snps.copy()
                index = len(qual_nooverlap_snps.keys())
                repeat = 0
                for overlap in overlaps:
                    if "print" in overlap.keys():
                        repeat += 1
                for times in range(1, len(overlaps) - repeat):
                    for key, value in qual_nooverlap_snps.items():
                        new_snps[key + index * times] = list(value)
                check_overlap(new_snps, overlaps)
                qual_nooverlap_snps = new_snps.copy()
        else:
            if "print" not in snp1.keys():
                if first:
                    qual_nooverlap_snps[num_overlap].append(snp1)
                    first = False
                else:
                    for key, value in qual_nooverlap_snps.items():
                        value.append(snp1)
                snp1["print"] = True
        if first:
            first = False
    return conflicts, qual_nooverlap_snps


def print_file(refs, out_ref, conflicts, key, values, mod_seq_init,
               mod_seqs, out_seq):
    num_seq = 1
    num_nt = 0
    paths = []
    if len(conflicts) == 0:
        paths.append("All")
    else:
        for conflict in conflicts:
            for path in conflict:
                for value in values:
                    if path == value:
                        paths.append(str(path["pos"]))
    if len(refs) != 0:
        for seq_name, ref_datas in refs.items():
            num_ref = 1
            for ref in ref_datas:
                out_ref.write("\t".join([str(key), "_".join(paths),
                              str(num_ref), ref, seq_name]) + "\n")
                num_ref += 1
    else:
        out_ref.write("\t".join([str(key), "_".join(paths), "1", "All",
                      mod_seq_init["genome"]]) + "\n")
    if len(mod_seqs) == 0:
        out_fasta = open("_".join([out_seq, mod_seq_init["genome"],
                                  str(key), "1.fa"]), "w")
        out_fasta.write(">{0}\n".format(mod_seq_init["genome"]))
        for nt in mod_seq_init["seq"]:
            num_nt += 1
            out_fasta.write("{0}".format(nt))
            if num_nt % 60 == 0:
                out_fasta.write("\n")
        out_fasta.close()
    else:
        for seq in mod_seqs:
            num_nt = 0
            out_fasta = open("_".join([out_seq, seq["genome"], str(key),
                                       str(num_seq)]) + ".fa", "w")
            out_fasta.write(">{0}\n".format(seq["genome"]))
            for nt in seq["seq"]:
                num_nt += 1
                out_fasta.write("{0}".format(nt))
                if num_nt % 60 == 0:
                    out_fasta.write("\n")
            num_seq += 1
            out_fasta.close()


def stat(max_quals, trans_snps, bam_number, stat_prefix,
         out_snp, args_snp, type_):
    out_stat = open("_".join([stat_prefix, type_]), "w")
    printed = False
    for strain, max_qual in max_quals.items():
        max_qual = int(((max_qual / 10) + 1) * 10)
        cutoffs = []
        if (strain == "All_strain") and (len(max_quals) > 2):
            printed = True
        elif (strain != "All_strain"):
            printed = True
        if printed:
            for cutoff in range(0, max_qual, 10):
                cutoffs.append(0)
            for snp in trans_snps:
                if (snp["strain"] == strain) or (strain == "All_strain"):
                    index = int(snp["qual"] / 10)
                    cutoffs[index] += 1
            num_cutoff = 10
            num_quality = 0
            out_stat.write(strain + ":\n")
            for cutoff in cutoffs:
                if args_snp.quality <= (num_cutoff - 10):
                    num_quality = num_quality + cutoff
                out_stat.write("the number of QUAL which is between "
                               "{0} and {1} = {2}\n".format(
                                   num_cutoff - 10, num_cutoff, cutoff))
                num_cutoff = num_cutoff + 10
            out_stat.write("the total numbers of QUAL which is "
                           "higher than {0} = {1}\n".format(
                               args_snp.quality, num_quality))
            plot_bar(cutoffs, strain, out_snp,
                     type_.replace(".csv", ".png"))
            printed = False
    out_stat.close()


def read_fasta(fasta_file):
    seqs = []
    first = True
    num_index = 0
    seq_name = ""
    with open(fasta_file, "r") as fh:
        for line in fh:
            line = line.strip()
            if line.startswith(">"):
                if first:
                    seqs.append({line[1:]: ""})
                    first = False
                else:
                    seqs.append({line[1:]: ""})
                    num_index += 1
                seq_name = line[1:]
            else:
                seqs[num_index][seq_name] = seqs[num_index][seq_name] + line
    return seqs


def gen_new_fasta(qual_nooverlap_snps, seqs, out_ref, conflicts, out_seq):
    refs = {}
    for key, values in qual_nooverlap_snps.items():
        for seq in seqs:
            for name in seq.keys():
                break
            refs[name] = []
            num_var = 0
            for strain, fasta in seq.items():
                mod_seq_init = {"genome": strain, "seq": fasta, "num_mod": 0}
                mod_seqs = []
            for snp in values:
                if snp["strain"] in seq.keys():
                    if "," in snp["alt"]:
                        num_var += 1
                        tmps = []
                        tmp_snps = []
                        alts = snp["alt"].split(",")
                        for alt in alts:
                            tmp_snp = snp.copy()
                            tmp_snp["alt"] = alt
                            tmp_snps.append(tmp_snp)
                            if len(mod_seqs) == 0:
                                num_mod_seqs = len(mod_seqs)
                                tmps.append(mod_seq_init.copy())
                            else:
                                num_mod_seqs = len(mod_seqs)
                                for mod_seq in mod_seqs:
                                    tmps.append(mod_seq.copy())
                        mod_seqs = list(tmps)
                        num_mod = 0
                        num = 1
                        refs[name] = gen_ref(tmp_snps, snp["pos"],
                                             refs[name], num_var)
                        for mod_seq in mod_seqs:
                            change(tmp_snps[num_mod], mod_seq)
                            if num >= num_mod_seqs:
                                num_mod += 1
                                num = 0
                            num += 1
                    else:
                        if len(mod_seqs) == 0:
                            change(snp, mod_seq_init)
                        else:
                            for mod_seq in mod_seqs:
                                change(snp, mod_seq)
        print_file(refs, out_ref, conflicts, key, values,
                   mod_seq_init, mod_seqs, out_seq)


def snp_detect(fasta_file, snp_file, depth_file, out_snp, out_seq,
               bam_number, stat_prefix, args_snp):
    max_quals, snps, dess, raw_snps = import_data(
            snp_file, args_snp, bam_number, depth_file)
    out_best = open(out_snp + "_best.vcf", "w")
    out_ref = open(out_snp + "_seq_reference.csv", "w")
    out_best.write("\n".join(dess) + "\n")
    out_best.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL"
                    "\tFILTER\tINFO\tFORMAT\tBAM\n")
    best_snps = []
    for snp in snps:
        if snp["qual"] >= args_snp.quality:
            out_best.write(snp["all_info"] + "\n")
            best_snps.append(snp)
    conflicts, qual_nooverlap_snps = overlap_position(best_snps)
    stat(max_quals, raw_snps, bam_number, stat_prefix, out_snp, args_snp, "raw.csv")
    stat(max_quals, snps, bam_number, stat_prefix, out_snp, args_snp, "best.csv")
    seqs = read_fasta(fasta_file)
    gen_new_fasta(qual_nooverlap_snps, seqs, out_ref, conflicts, out_seq)
    out_best.close()
    out_ref.close()



import csv


def rbs_from_rfam(ribo_table, rfam_file, out_file):
    ribos = []
    out = open(out_file, "w")
    f_h = open(ribo_table, "r")
    for row in csv.reader(f_h, delimiter="\t"):
        if not row[0].startswith("#"):
            ribos.append(row[0].strip())
    detect = False
    with open(rfam_file, "r") as r_h:
        for line in r_h:
            line = line.rstrip("\n")
            datas = line.split(" ")
            if ("INFERNAL" in datas[0]) or (
                    "HMMER" in datas[0]):
                header = line
                detect = False
            elif "NAME" in datas[0]:
                name = line
            elif ("ACC" in datas[0]):
                for ribo in ribos:
                    if datas[-1] == ribo:
                        out.write("{0}\n{1}\n{2}\n".format(header, name, line))
                        detect = True
            else:
                if (detect):
                    out.write(line + "\n")
    out.close()
    f_h.close()



import os
from annogesiclib.helper import Helper
from annogesiclib.gff3 import Gff3Parser


def assign_name(entry):
    if entry.attributes["locus_tag"] == entry.attributes["Name"]:
        return None
    else:
        return entry.attributes["Name"]

def print_fasta(entry, seq, out, gene):
    if gene is not None:
        if ("locus_tag" in gene.attributes.keys()):
            locus = gene.attributes["locus_tag"]
        else:
            locus = "NA"
    else:
        locus = "NA"
    if ("ID" in entry.attributes.keys()):
        out.write(">{0}|{1}-{2}_{3}\n{4}\n".format(
                  "|".join([locus,
                            entry.attributes["ID"]]),
                  entry.start, entry.end,
                  entry.strand, seq))
    else:
        out.write(">{0}|{1}-{2}_{3}\n{4}\n".format(
                  "|".join([locus, "NA"]) , entry.start, entry.end,
                  entry.strand, seq))


def read_file(seq_file, gff_file, target_folder, features):
    fastas = []
    cdss_f = []
    cdss_r = []
    genes = []
    with open(seq_file, "r") as seq_f:
        for line in seq_f:
            if line.startswith(">"):
                continue
            else:
                line = line.strip()
                fastas.append(line)
    fasta = "".join(fastas)
    g_h = open(gff_file)
    for entry in Gff3Parser().entries(g_h):
        if os.path.exists(os.path.join(target_folder,
                          "_".join([entry.seq_id, "target.fa"]))):
            os.remove(os.path.join(target_folder,
                      "_".join([entry.seq_id, "target.fa"])))
        for feature in features:
            if (entry.feature == feature) and (entry.strand == "+"):
                cdss_f.append(entry)
            elif (entry.feature == feature) and (entry.strand == "-"):
                cdss_r.append(entry)
        if entry.feature == "gene":
            genes.append(entry)
    g_h.close()
    return fasta, cdss_f, cdss_r, genes


def check_parent_gene(cds, genes):
    target_gene = None
    for gene in genes:
        if "Parent" in cds.attributes.keys():
            if (gene.attributes["ID"] in
                    cds.attributes["Parent"].split(",")):
                target_gene = gene
                break
        else:
            if (gene.seq_id == cds.seq_id) and (
                    gene.strand == cds.strand):
                if ((cds.start <= gene.start) and (
                        cds.end >= gene.end)) or (
                        (cds.start >= gene.start) and (
                        cds.end <= gene.end)) or (
                        (cds.start <= gene.start) and (
                        cds.end <= gene.end) and (
                        cds.end >= gene.start)) or (
                        (cds.start >= gene.start) and (
                        cds.start <= gene.end) and (
                        cds.end >= gene.end)):
                    target_gene = gene
                    break
    return target_gene


def deal_cds_forward(cdss_f, target_folder, fasta, genes, tar_start, tar_end):
    '''for forward strand'''
    pre_id = ""
    out = None
    for cds in cdss_f:
        if cds.seq_id != pre_id:
            out = open(os.path.join(target_folder,
                       "_".join([cds.seq_id, "target.fa"])), "w")
            pre_id = cds.seq_id
        if (cds.start > tar_start):
            start = cds.start - tar_start
        else:
            start = 1
        if ((cds.start + tar_end) < len(fasta)) and (
                (cds.end - cds.start) >= tar_end):
            end = cds.start + tar_end - 1
        elif cds.start + tar_end >= len(fasta):
            end = len(fasta)
        elif (cds.end - cds.start) < tar_end:
            end = cds.end
        seq = Helper().extract_gene(fasta, start, end, cds.strand)
        target = cds
        target_gene = check_parent_gene(cds, genes)
        print_fasta(target, seq, out, target_gene)
    if out is not None:
        out.close()


def deal_cds_reverse(cdss_r, target_folder, fasta, genes, tar_start, tar_end):
    '''for the reverse strand'''
    pre_id = ""
    out = None
    for cds in cdss_r:
        if cds.seq_id != pre_id:
            out = open(os.path.join(target_folder,
                       "_".join([cds.seq_id, "target.fa"])), "a")
            pre_id = cds.seq_id
        if (len(fasta) - cds.end > tar_start):
            end = cds.end + tar_start
        else:
            end = len(fasta)
        if ((cds.end - tar_end) > 1) and ((cds.end - cds.start) >= tar_end):
            start = cds.end - tar_end - 1
        elif cds.end - tar_end < 1:
            start = 1
        elif (cds.end - cds.start) < tar_end:
            start = cds.start
        seq = Helper().extract_gene(fasta, start, end, cds.strand)
        target = cds
        target_gene = check_parent_gene(cds, genes)
        print_fasta(target, seq, out, target_gene)
    if out is not None:
        out.close()


def potential_target(gff_file, seq_file, target_folder, args_tar):
    '''get the sequence of the potential target of sRNA'''
    fasta, cdss_f, cdss_r, genes = read_file(seq_file, gff_file,
                                             target_folder, args_tar.features)
    sort_cdss_f = sorted(cdss_f, key=lambda k: (k.seq_id, k.start,
                                                k.end, k.strand))
    deal_cds_forward(sort_cdss_f, target_folder, fasta, genes,
                     args_tar.tar_start, args_tar.tar_end)
    sort_cdss_r = sorted(cdss_r, reverse=True,
                         key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    deal_cds_reverse(sort_cdss_r, target_folder, fasta, genes,
                     args_tar.tar_start, args_tar.tar_end)



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def del_attributes(entry, features):
    attributes = {}
    for key, value in entry.attributes.items():
        if (key not in features):
            attributes[key] = value
    return attributes


def comparing(ta, ter, fuzzy_down_ta, fuzzy_up_ta, stats):
    '''main part for comparing terminator and transcript'''
    if (ta.seq_id == ter.seq_id) and (
            ta.strand == ter.strand):
        if ta.strand == "+":
            if ((ta.end >= ter.start) and (
                     ta.end <= ter.end)) or (
                    (ta.end <= ter.start) and (
                     (ter.start - ta.end) <= fuzzy_down_ta)) or (
                    (ta.end >= ter.end) and (
                     (ta.end - ter.end) <= fuzzy_up_ta)) or (
                    (ta.start <= ter.start) and (
                     ta.end >= ter.end)):
                if ter.attributes["Parent"] == "NA":
                    stats[ta.seq_id]["overlap"] += 1
                    ta.attributes["associated_term"] = (
                        "terminator:" + str(ter.start) + "-" +
                        str(ter.end) + "_" + ter.strand)
                    if "ID" in ta.attributes.keys():
                        ter.attributes["Parent"] = ta.attributes["ID"]
                    else:
                        ter.attributes["Parent"] = (
                            "transcript:" + str(ta.start) + "-" +
                            str(ta.end) + "_" + ta.strand)
        else:
            if ((ta.start >= ter.start) and (
                     ta.start <= ter.end)) or (
                    (ta.start <= ter.start) and (
                     (ter.start - ta.start) <= fuzzy_up_ta)) or (
                    (ta.start >= ter.end) and (
                     (ta.start - ter.end) <= fuzzy_down_ta)) or (
                    (ta.start <= ter.start) and (
                     ta.end >= ter.end)):
                if ter.attributes["Parent"] == "NA":
                    stats[ta.seq_id]["overlap"] += 1
                ta.attributes["associated_term"] = (
                    "terminator:" + str(ter.start) + "-" +
                    str(ter.end) + "_" + ter.strand)
                if "ID" in ta.attributes.keys():
                    ter.attributes["Parent"] = ta.attributes["ID"]
                else:
                    ter.attributes["Parent"] = (
                        "transcript:" + str(ta.start) + "-" +
                        str(ta.end) + "_" + ta.strand)


def output_term(ters, term_file, type_, term_outfolder):
    out = open(term_file + "tmp", "w")
    out.write("##gff-version 3\n")
    for ter in ters:
        attribute_string = ";".join(
            ["=".join(items) for items in ter.attributes.items()])
        out.write("\t".join([ter.info_without_attributes,
                             attribute_string]) + "\n")
    out.close()
    os.remove(term_file)
    filename = term_file.split("/")[-1]
    if filename in os.listdir(term_outfolder):
        os.remove(os.path.join(term_outfolder, filename))
    shutil.copy(term_file + "tmp", term_file)
    shutil.move(term_file + "tmp", os.path.join(term_outfolder, filename))
    if type_ == "terminator":
        table_file = term_file.replace("/gffs/", "/tables/")
        table_file = table_file.replace(".gff", ".csv")
        out_t = open(table_file + "tmp", "w")
        out_t.write("\t".join(["strain", "name", "start", "end", "strand",
                               "detect", "associated_gene",
                               "associated_transcript",
                               "coverage_decrease", "coverage_detail"]) + "\n")
        fh = open(table_file, "r")
        for row in csv.reader(fh, delimiter='\t'):
            if row[0] != "strain":
                for ter in ters:
                    if (row[0] == ter.seq_id) and (
                            row[2] == str(ter.start)) and (
                            row[3] == str(ter.end)) and (
                            row[4] == ter.strand):
                        out_t.write("\t".join([row[0], row[1], row[2], row[3],
                                    row[4], row[5], row[6],
                                    ter.attributes["Parent"],
                                    row[7], row[8]]) + "\n")
                        break
        fh.close()
        out_t.close()
        os.remove(table_file)
        shutil.move(table_file + "tmp", table_file)


def read_gff(filename, index):
    gf = open(filename, "r")
    gff_parser = Gff3Parser()
    datas = []
    for entry in gff_parser.entries(gf):
        entry.attributes[index] = "NA"
        datas.append(entry)
        datas = sorted(datas, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
    gf.close()
    return datas


def compare_term_tran(trans, terms, fuzzy_up_ta, fuzzy_down_ta,
                      out_folder, type_, term_outfolder, tran_outfolder):
    '''Comparison of terminator and transcript. It can realise the 
    relationship of terminator and transcript'''
    for tran in os.listdir(trans):
        if tran.endswith("_transcript.gff"):
            prefix = tran.replace("_transcript.gff", "")
            out_g = open(os.path.join(trans, tran) + "tmp", "w")
            out_g.write("##gff-version 3\n")
            tas = read_gff(os.path.join(trans, tran), "associated_term")
            ters = read_gff(os.path.join(terms, prefix + "_term.gff"),
                            "Parent")
            stats = {}
            pre_seq = ""
            for ta in tas:
                if ta.seq_id != pre_seq:
                    stats[ta.seq_id] = {"all_tran": 0, "all_term": 0,
                                        "overlap": 0}
                    pre_seq = ta.seq_id
                    new_term = True
                stats[ta.seq_id]["all_tran"] += 1
                for ter in ters:
                    if new_term:
                        stats[ta.seq_id]["all_term"] += 1
                    comparing(ta, ter, fuzzy_down_ta, fuzzy_up_ta, stats)
                if new_term:
                    new_term = False
                attribute_string = ";".join(
                    ["=".join(items) for items in ta.attributes.items()])
                out_g.write("\t".join([ta.info_without_attributes,
                                       attribute_string]) + "\n")
            os.remove(os.path.join(trans, tran))
            if tran in os.listdir(tran_outfolder):
                os.remove(os.path.join(tran_outfolder, tran))
            shutil.copy(os.path.join(trans, tran) + "tmp",
                        os.path.join(tran_outfolder, tran))
            shutil.move(os.path.join(trans, tran) + "tmp",
                        os.path.join(trans, tran))
            output_term(ters, os.path.join(terms, prefix + "_term.gff"), type_,
                        term_outfolder)
            out = open(os.path.join(out_folder,
                       "statistics/stat_compare_transcript_terminator_" + prefix + ".csv"), "w")
            for strain, stat in stats.items():
                out.write(strain + ":\n")
                out.write("\tThe overlap between transcripts "
                          "and terminators are {0}\n".format(
                           stat["overlap"]))
                out.write("\tThe overlap percentage of transcripts are {0}\n".format(
                          float(stat["overlap"])/float(stat["all_tran"])))
                out.write("\tThe overlap percentage of terminators are {0}\n".format(
                          float(stat["overlap"])/float(stat["all_term"])))
            out.close()



from annogesiclib.gff3 import Gff3Parser


def modify_position(frag, norm):
    '''get proper position, we choose the long one'''
    if frag.end < norm.end:
        frag.end = norm.end
    if frag.start > norm.start:
        frag.start = norm.start
    norm.attributes["print"] = True
    frag.attributes["print"] = True


def print_file(data, out, name, num):
    attributes = {}
    attributes["ID"] = data.seq_id + "_transcript" + str(num)
    attributes["Name"] = "transcript_" + name
    attributes["detect_lib"] = data.attributes["detect_lib"]
    attribute_string = ";".join(["=".join(items)
                                 for items in attributes.items()])
    out.write("\t".join([str(field) for field in [
                        data.seq_id, data.source, data.feature, data.start,
                        data.end, data.score, data.strand, data.phase,
                        attribute_string]]) + "\n")


def store(data, source, finals):
    data.attributes["detect_lib"] = source
    data.attributes["print"] = False
    finals.append(data)


def compare(data1, data2, overlap, tolerance):
    '''search the sRNA which can be detected in frag and tex libs.
    Then, try to merge them to be a longer one'''
    if (data1.seq_id == data2.seq_id) and (data1.strand == data2.strand):
        if (data1.start <= (data2.end + tolerance)) and (
                data1.start >= data2.start) and (
                data1.end >= (data2.end + tolerance)):
            modify_position(data1, data2)
            overlap = True
        elif (data1.end >= (data2.start - tolerance)) and (
                data1.end <= data2.end) and (
                data1.start <= (data2.start - tolerance)):
            modify_position(data1, data2)
            overlap = True
        elif (data1.start <= data2.start) and (
                data1.end >= data2.end):
            modify_position(data1, data2)
            overlap = True
        elif (data2.start <= data1.start) and (
                data2.end >= data1.end):
            modify_position(data1, data2)
            overlap = True
    return overlap


def combine(frag_file, tex_file, tolerance, output_file):
    '''merge the results of sRNA which detected by fragmented and dRNA'''
    frags = []
    norms = []
    finals = []
    out = open(output_file, "w")
    out.write("##gff-version 3\n")
    f_h = open(frag_file, "r")
    for entry in Gff3Parser().entries(f_h):
        entry.attributes["print"] = False
        frags.append(entry)
    f_h.close()
    n_h = open(tex_file, "r")
    for entry in Gff3Parser().entries(n_h):
        entry.attributes["print"] = False
        norms.append(entry)
    n_h.close()
    sort_frags = sorted(frags, key=lambda k: (k.seq_id, k.start,
                                              k.end, k.strand))
    sort_norms = sorted(norms, key=lambda k: (k.seq_id, k.start,
                                              k.end, k.strand))
    for frag in sort_frags:
        overlap = False
        for norm in sort_norms:
            overlap = compare(frag, norm, overlap, tolerance)
        if overlap:
            store(frag, "fragmented,tex_notex", finals)
        else:
            store(frag, "fragmented", finals)
    for norm in sort_norms:
        if not norm.attributes["print"]:
            store(norm, "tex_notex", finals)
    sort_finals = sorted(finals, key=lambda k: (k.seq_id, k.start,
                                                k.end, k.strand))
    num = 0
    for tar in sort_finals:
        if tar.attributes["print"]:
            continue
        overlap = False
        for ref in sort_finals:
            overlap = compare(tar, ref, overlap, tolerance)
        name = '%0*d' % (5, num)
        print_file(tar, out, name, num)
        num += 1
    out.close()



class SeqModifier(object):
    """Help to apply SNPs, insertion and deletions to a sequence."""

    def __init__(self, seq):
        self._seq = seq
        self._init_pos_dict()

    def seq(self):
        return self._seq

    def _init_pos_dict(self):
        self._org_pos_to_internal_pos = dict(
            [(pos, pos-1)
             for pos in range(1, len(self._seq) + 1)])

    def replace(self, pos, nucleotide):
        seq_as_list = list(self._seq)
        seq_as_list[self._org_pos_to_internal_pos[pos]] = nucleotide
        self._seq = "".join(seq_as_list)

    def remove(self, pos, num):
        int_pos = self._org_pos_to_internal_pos[pos]
        self._seq = self._seq[:int_pos] + self._seq[int_pos+1:]
        del(self._org_pos_to_internal_pos[pos])
        for pos in range(pos, len(self._seq) + 2):
            try:
                self._org_pos_to_internal_pos[pos] = (
                    self._org_pos_to_internal_pos[pos] - num)
            except KeyError:
                pass

    def insert(self, pos, nucleotide):
        """Insert after nucleotide of the given position"""
        int_pos = self._org_pos_to_internal_pos[pos]
        self._seq = self._seq[:int_pos] + nucleotide + self._seq[int_pos:]
        for pos in range(pos + 1, len(self._seq) + 1):
            try:
                self._org_pos_to_internal_pos[pos] = (
                    self._org_pos_to_internal_pos[pos] + len(nucleotide))
            except KeyError:
                pass

    def get_nucl(self, pos):
        return self._seq[self._org_pos_to_internal_pos[pos]]



import os
import sys
import shutil
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.TSS_upstream import upstream, del_repeat_fasta
from annogesiclib.gen_promoter_table import gen_promoter_table


class MEME(object):
    '''detection of promoter'''

    def __init__(self, args_pro):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.tss_path = os.path.join(args_pro.tsss, "tmp")
        if args_pro.gffs is not None:
            self.gff_path = os.path.join(args_pro.gffs, "tmp")
        else:
            self.gff_path = None
        self.out_fasta = os.path.join(args_pro.output_folder, "fasta_class")
        self.tmp_folder = os.path.join(os.getcwd(), "tmp")
        self.fastas = {"pri": os.path.join(self.tmp_folder, "primary.fa"),
                       "sec": os.path.join(self.tmp_folder, "secondary.fa"),
                       "inter": os.path.join(self.tmp_folder, "internal.fa"),
                       "anti": os.path.join(self.tmp_folder, "antisense.fa"),
                       "orph": os.path.join(self.tmp_folder, "orphan.fa"),
                       "all_no_orph": "without_orphan.fa",
                       "all": "all_type.fa",
                       "tmp_fa": os.path.join(self.tmp_folder, "tmp.fa"),
                       "tmp_all": os.path.join(self.tmp_folder, "tmp_all.fa")}
        self.all_fasta = os.path.join(args_pro.fastas, "allfasta.fa")
        self.all_tss = os.path.join(self.tss_path, "allfasta_TSS.gff")

    def _gen_and_check_folder(self, out_path, folder, type_):
        sub_out_folder = os.path.join(out_path, type_)
        if folder in os.listdir(sub_out_folder):
            shutil.rmtree(os.path.join(sub_out_folder, folder))
        return sub_out_folder

    def _run_normal_motif(self, input_path, out_path, filename,
                          fasta, width, args_pro):
        '''run MEME with specific width'''
        folder = "_".join(["promoter_motifs", filename,
                           str(width), "nt"])
        if (args_pro.program.lower() == "meme") or (
                args_pro.program.lower() == "both"):
            meme_folder = self._gen_and_check_folder(
                            out_path, folder, "MEME")
            command = [args_pro.meme_path, "-maxsize", "1000000",
                       "-dna", "-nmotifs", str(args_pro.num_motif),
                       "-w", str(width), "-maxiter", "100",
                       "-evt", str(args_pro.e_value)]
            if args_pro.para is not None:
                command = command + ["-p", args_pro.para]
            call(command + ["-oc", os.path.join(meme_folder, folder),
                            os.path.join(input_path, fasta)])
        if (args_pro.program.lower() == "glam2") or (
                args_pro.program.lower() == "both"):
            glam_folder = self._gen_and_check_folder(
                            out_path, folder, "GLAM2")
            call([args_pro.glam2_path,
                  "-O", os.path.join(glam_folder, folder), "-w",
                  str(width), "-b", str(width), "-r",
                  str(args_pro.num_motif), "-n", str(args_pro.end_run),
                  "n", os.path.join(input_path, fasta)])

    def _run_small_motif(self, input_path, out_path, filename,
                         fasta, width, args_pro):
        '''run MEME with range of width'''
        data = width.split("-")
        min_width = data[0]
        max_width = data[1]
        folder = "_".join(["promoter_motifs", filename,
                           "-".join([str(min_width), str(max_width)]), "nt"])
        if (args_pro.program.lower() == "meme") or (
                args_pro.program.lower() == "both"):
            meme_folder = self._gen_and_check_folder(
                            out_path, folder, "MEME")
            command = [args_pro.meme_path, "-maxsize", "1000000",
                       "-dna", "-nmotifs", str(args_pro.num_motif),
                       "-minsites", "0", "-maxsites", "2",
                       "-minw", str(min_width), "-maxw", str(max_width),
                       "-maxiter", "100",
                       "-evt", str(args_pro.e_value)]
            if args_pro.para is not None:
                command = command + ["-p", args_pro.para]
            call(command + ["-oc", os.path.join(meme_folder, folder),
                            os.path.join(input_path, fasta)])
        elif (args_pro.program.lower() == "glam2") or (
                args_pro.program.lower() == "both"):
            glam_folder = self._gen_and_check_folder(
                            out_path, folder, "GLAM2")
            call([args_pro.glam2_path,
                  "-O", os.path.join(glam_folder, folder), "-a",
                  str(min_width), "-b", str(max_width), "-r",
                  str(args_pro.num_motif), "-n", str(args_pro.end_run),
                  "n", os.path.join(input_path, fasta)])

    def _get_fasta_file(self, fasta_path, prefix):
        for fasta in os.listdir(fasta_path):
            if (fasta.endswith(".fa")) and \
               (prefix == fasta.replace(".fa", "")):
                break
            elif (fasta.endswith(".fna")) and \
                 (prefix == fasta.replace(".fna", "")):
                break
            elif (fasta.endswith(".fasta")) and \
                 (prefix == fasta.replace(".fasta", "")):
                break
        return fasta

    def _check_gff(self, gffs):
        for gff in os.listdir(gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(gffs, gff))

    def _move_and_merge_fasta(self, input_path, prefix):
        all_type = os.path.join(self.tmp_folder, self.fastas["all"])
        all_no_orph = os.path.join(self.tmp_folder, self.fastas["all_no_orph"])
        if self.fastas["all"] in os.listdir(self.tmp_folder):
            os.remove(all_type)
        if self.fastas["all_no_orph"] in os.listdir(self.tmp_folder):
            os.remove(all_no_orph)
        shutil.copyfile(self.fastas["pri"], self.fastas["tmp_fa"])
        self.helper.merge_file(self.fastas["sec"], self.fastas["tmp_fa"])
        self.helper.merge_file(self.fastas["inter"], self.fastas["tmp_fa"])
        self.helper.merge_file(self.fastas["anti"], self.fastas["tmp_fa"])
        shutil.copyfile(self.fastas["tmp_fa"], self.fastas["tmp_all"])
        self.helper.merge_file(self.fastas["orph"], self.fastas["tmp_all"])
        del_repeat_fasta(self.fastas["tmp_fa"], all_no_orph)
        del_repeat_fasta(self.fastas["tmp_all"], all_type)
        os.remove(self.fastas["tmp_fa"])
        os.remove(self.fastas["tmp_all"])
        out_prefix = os.path.join(input_path, prefix)
        shutil.move(self.fastas["pri"], "_".join([
            out_prefix, "allstrain_primary.fa"]))
        shutil.move(self.fastas["sec"], "_".join([
            out_prefix, "allstrain_secondary.fa"]))
        shutil.move(self.fastas["inter"], "_".join([
            out_prefix, "allstrain_internal.fa"]))
        shutil.move(self.fastas["anti"], "_".join([
            out_prefix, "allstrain_antisense.fa"]))
        shutil.move(self.fastas["orph"], "_".join([
            out_prefix, "allstrain_orphan.fa"]))
        shutil.move(all_type, "_".join([
            out_prefix, "allstrain_all_types.fa"]))
        shutil.move(all_no_orph, "_".join([
            out_prefix, "allstrain_without_orphan.fa"]))

    def _split_fasta_by_strain(self, input_path):
        for fasta in os.listdir(input_path):
            if "allstrain" not in fasta:
                os.remove(os.path.join(input_path, fasta))
        out = None
        for fasta in os.listdir(input_path):
            if fasta.endswith(".fa"):
                pre_strain = ""
                num_strain = 0
                with open(os.path.join(input_path, fasta), "r") as f_h:
                    for line in f_h:
                        line = line.strip()
                        if line.startswith(">"):
                            datas = line.split("_")
                            strain = "_".join(datas[2:])
                            if pre_strain != strain:
                                num_strain += 1
                                filename = fasta.split("allstrain")
                                if out is not None:
                                    out.close()
                                out = open(os.path.join(
                                           input_path, "".join([
                                               filename[0], strain,
                                               filename[-1]])), "a")
                                pre_strain = strain
                            out.write(line + "\n")
                        else:
                            out.write(line + "\n")
                if num_strain <= 1:
                    os.remove(os.path.join(input_path,
                              "".join([filename[0], strain, filename[-1]])))
        out.close()

    def _run_program(self, prefixs, args_pro):
        for prefix in prefixs:
            input_path = os.path.join(self.out_fasta, prefix)
            out_path = os.path.join(args_pro.output_folder, prefix)
            if args_pro.program.lower() == "both":
                self.helper.check_make_folder(os.path.join(out_path, "MEME"))
                self.helper.check_make_folder(os.path.join(out_path, "GLAM2"))
            elif args_pro.program.lower() == "meme":
                self.helper.check_make_folder(os.path.join(out_path, "MEME"))
            elif args_pro.program.lower() == "glam2":
                self.helper.check_make_folder(os.path.join(out_path, "GLAM2"))
            for fasta in os.listdir(input_path):
                filename = fasta.replace(".fa", "")
                for width in args_pro.widths:
                    print("Computing promoters of {0} - {1}".format(
                          fasta, width))
                    if "-" in width:
                        self._run_small_motif(input_path, out_path, filename,
                                              fasta, width, args_pro)
                    else:
                        self._run_normal_motif(input_path, out_path, filename,
                                               fasta, width, args_pro)

    def _combine_file(self, prefixs, args_pro):
        '''combine all TSS file in the input folder to generate the 
        global TSS for detecting the global promoter'''
        if args_pro.source:
            for tss in os.listdir(self.tss_path):
                if tss.endswith("_TSS.gff"):
                    self.helper.merge_file(os.path.join(
                         self.tss_path, tss), self.all_tss)
            for fasta in os.listdir(args_pro.fastas):
                if (fasta.endswith(".fa")) or (
                        fasta.endswith(".fna")) or (
                        fasta.endswith(".fasta")):
                    self.helper.merge_file(os.path.join(
                         args_pro.fastas, fasta), self.all_fasta)
        else:
            for tss in os.listdir(os.path.join(
                                  args_pro.output_folder, "TSS_class")):
                if tss.endswith("_TSS.gff"):
                    self.helper.merge_file(os.path.join(
                         self.tss_path, tss), self.all_tss)
            for fasta in os.listdir(args_pro.fastas):
                if (fasta.endswith(".fa")) or (
                        fasta.endswith(".fna")) or (
                        fasta.endswith(".fasta")):
                    self.helper.merge_file(os.path.join(
                         args_pro.fastas, fasta), self.all_fasta)
        print("Generating fasta file of all fasta files")
        prefixs.append("allfasta")
        input_path = os.path.join(self.out_fasta, "allfasta")
        self.helper.check_make_folder(os.path.join(
                                      args_pro.output_folder, "allfasta"))
        self.helper.check_make_folder(os.path.join(
                                      self.out_fasta, "allfasta"))
        args_pro.source = True
        upstream(self.all_tss, self.all_fasta, None,
                 None, args_pro, None)
        self._move_and_merge_fasta(input_path, "allfasta")

    def _remove_files(self, args_pro):
        self.helper.remove_tmp_dir(args_pro.fastas)
        self.helper.remove_tmp_dir(args_pro.tsss)
        self.helper.remove_tmp_dir(args_pro.gffs)
        if "tmp_wig" in os.listdir(args_pro.output_folder):
            shutil.rmtree(os.path.join(args_pro.output_folder, "tmp_wig"))
        if "allfasta" in os.listdir(os.getcwd()):
            shutil.rmtree("allfasta")
        shutil.rmtree("tmp")

    def _gen_table(self, output_folder, prefixs, combine, program):
        '''generate the promoter table'''
        if combine:
            strains = prefixs + ["allfasta"]
        else:
            strains = prefixs
        for strain in strains:
            tss_file = os.path.join(self.tss_path, strain + "_TSS.gff")
            if (program.lower() == "both") or (
                    program.lower() == "meme"):
                for folder in os.listdir(os.path.join(output_folder,
                                                      strain, "MEME")):
                    gen_promoter_table(os.path.join(output_folder, strain,
                                       "MEME", folder, "meme.txt"),
                                       os.path.join(output_folder, strain,
                                       "MEME", folder, "meme.csv"), tss_file,
                                       "meme")
            if (program.lower() == "both") or (
                    program.lower() == "glam2"):
                for folder in os.listdir(os.path.join(output_folder,
                                                      strain, "GLAM2")):
                    gen_promoter_table(os.path.join(output_folder, strain,
                                        "GLAM2", folder, "glam2.txt"),
                                        os.path.join(output_folder, strain,
                                        "GLAM2", folder, "glam2.csv"),
                                        tss_file, "glam2")

    def _get_upstream(self, args_pro, prefix, tss, fasta):
        '''get upstream sequence of TSS'''
        if args_pro.source:
            print("Generating fasta file of {0}".format(prefix))
            upstream(os.path.join(self.tss_path, tss),
                     os.path.join(args_pro.fastas, fasta),
                     None, None, args_pro, prefix)
        else:
            if (args_pro.gffs is None) or (
                    args_pro.tex_wigs is None) or (
                    args_pro.input_libs is None):
                print("Error: Please assign proper annotation, tex +/- "
                      "wig folder and tex treated libs!!!")
                sys.exit()
            if "TSS_class" not in os.listdir(args_pro.output_folder):
                os.mkdir(os.path.join(args_pro.output_folder, "TSS_class"))
            
            print("Classifying TSS and extracting fasta {0}".format(prefix))
            upstream(os.path.join(self.tss_path, tss),
                     os.path.join(args_pro.fastas, fasta),
                     os.path.join(self.gff_path, prefix + ".gff"),
                     os.path.join(args_pro.output_folder, "TSS_class",
                     "_".join([prefix, "TSS.gff"])), args_pro, prefix)

    def run_meme(self, args_pro):
        if "allfasta.fa" in os.listdir(args_pro.fastas):
            os.remove(self.all_fasta)
            if "allfasta.fa_folder" in os.listdir(args_pro.fastas):
                shutil.rmtree(os.path.join(args_pro.fastas,
                              "allfasta.fa_folder"))
        self.multiparser.parser_fasta(args_pro.fastas)
        self.multiparser.parser_gff(args_pro.tsss, "TSS")
        if "allfasta_TSS.gff" in os.listdir(self.tss_path):
            os.remove(self.all_tss)
        if args_pro.gffs is not None:
            self._check_gff(args_pro.gffs)
            self.multiparser.parser_gff(args_pro.gffs, None)
            self.multiparser.combine_gff(args_pro.fastas, self.gff_path,
                                         "fasta", None)
        self._check_gff(args_pro.tsss)
        self.multiparser.combine_gff(args_pro.fastas, self.tss_path,
                                     "fasta", "TSS")
        self.helper.check_make_folder(self.out_fasta)
        self.helper.check_make_folder(self.tmp_folder)
        prefixs = []
        for tss in os.listdir(self.tss_path):
            prefix = tss.replace("_TSS.gff", "")
            prefixs.append(prefix)
            self.helper.check_make_folder(os.path.join(args_pro.output_folder,
                                                       prefix))
            self.helper.check_make_folder(os.path.join(self.out_fasta,
                                                       prefix))
            input_path = os.path.join(self.out_fasta, prefix)
            fasta = self._get_fasta_file(args_pro.fastas, prefix)
            self._get_upstream(args_pro, prefix, tss, fasta)
            self._move_and_merge_fasta(input_path, prefix)
            self._split_fasta_by_strain(input_path)
        if args_pro.combine:
            self._combine_file(prefixs, args_pro)
        self._run_program(prefixs, args_pro)
        print("Generating the table")
        self._gen_table(args_pro.output_folder, prefixs,
                        args_pro.combine, args_pro.program)
        self._remove_files(args_pro)



import os
import csv
import shutil


def mapping_ribos(table_folder, id_file, feature):
    ids = []
    ih = open(id_file, "r")
    for row in csv.reader(ih, delimiter='\t'):
        if not row[0].startswith("#"):
            ids.append({"id": row[0].strip(),
                        "name": row[1].strip(),
                        "info": row[2].strip()})
    for table_file in os.listdir(table_folder):
        if table_file.endswith("_" + feature + ".csv"):
            tmp_table = os.path.join(table_folder, "tmp" + table_file)
            table_file = os.path.join(table_folder, table_file)
            out = open(tmp_table, "w")
            tables = []
            fh = open(table_file, "r")
            out.write("#ID\tstrain\tstrand\tassociated_CDS\tstart_genome\t"
                      "end_genome\tRfam\te_value\tstart_align\tend_align\n")
            for row in csv.reader(fh, delimiter='\t'):
                if not row[0].startswith("#"):
                    tables.append({"input": row[0:6], "Rfam": row[6],
                                   "e": row[7], "start": row[8],
                                   "end": row[9]})
            for table in tables:
                for id_ in ids:
                    if table["Rfam"] == id_["id"]:
                        name = id_["name"]
                out.write("\t".join(table["input"] + [table["Rfam"], name,
                                    table["e"], table["start"],
                                    table["end"]]) + "\n")
            out.close()
            os.remove(table_file)
            shutil.move(tmp_table, table_file)



import os
import sys


class ProjectCreator(object):

    def create_root_folder(self, project_name):
        """Create the root folder of a new project with the given name.
        Arguments:
        - `project_name`: Name of the project root folder
        """
        if not os.path.exists(project_name):
            os.mkdir(project_name)
        else:
            sys.stderr.write("Cannot create folder \"%s\"! File/folder with "
                             "the same name exists already.\n" % project_name)
            sys.exit(2)

    def create_subfolders(self, subfolders):
        """Create required subfolders in the given folder.
        Arguments:
        - `project_name`: Name of the project root folder
        """
        for folder in subfolders:
            if not os.path.exists(folder):
                os.mkdir(folder)

    def create_version_file(self, version_file_path, version):
        with open(version_file_path, "w") as fh:
            fh.write("ANNOgesic version %s" % version)



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def compare_srna_gff(gffs, strain, strand, start, end, srna_types, file_type):
    for gff in gffs:
        if (strain == gff.seq_id) and (
                strand != gff.strand):
            if ((start <= gff.start) and (
                     end >= gff.end)) or (
                    (start >= gff.start) and (
                     end <= gff.end)) or (
                    (start <= gff.start) and (
                     end <= gff.end) and (
                     end >= gff.start)) or (
                    (start >= gff.start) and (
                     start <= gff.end) and (
                     end >= gff.end)):
                if file_type == "gff":
                    if "antisense" not in srna_types:
                        srna_types = srna_types + "," + "antisense"
                else:
                    if "Antisense" not in srna_types:
                        srna_types = srna_types + "," + "Antisense"
    return srna_types


def srna_antisense(srna_gff, srna_table, gff_file):
    tmp_srna_gff = srna_gff + "tmp"
    tmp_srna_table = srna_table + "tmp"
    out = open(tmp_srna_gff, "w")
    out.write("##gff-version 3\n")
    out_t = open(tmp_srna_table, "w")
    out_t.write("\t".join(["rank", "strain", "name", "start", "end", "strand",
                           "start_with_TSS/Cleavage_site", "end_with_cleavage",
                           "candidates", "lib_type", "best_avg_coverage",
                           "best_highest_coverage", "best_lower_coverage",
                           "track/coverage",
                           "normalized_secondary_energy_change(by_length)",
                           "sRNA_types", "confliction_of_sORF",
                           "nr_hit_number", "sRNA_hit_number",
                           "nr_hit_top3|ID|e-value", "sRNA_hit|e-value",
                           "overlap_CDS", "overlap_percent",
                           "end_with_terminator"]) + "\n")
    srnas = []
    sf = open(srna_gff, "r")
    for entry in Gff3Parser().entries(sf):
        srnas.append(entry)
    tabs = []
    fh = open(srna_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        if row[0] != "rank":
            tabs.append({"info": row, "strain": row[1], "strand": row[5],
                         "start": int(row[3]), "end": int(row[4]),
                         "srna_type": row[15]})
        else:
            out_t.write("\t".join(row) + "\n")
    gffs = []
    gf = open(gff_file, "r")
    for entry in Gff3Parser().entries(gf):
        gffs.append(entry)
    for srna in srnas:
        compare_srna_gff(gffs, srna.seq_id, srna.strand, srna.start, srna.end,
                         srna.attributes["sRNA_type"], "gff")
        attribute_string = ";".join(
            ["=".join(items) for items in srna.attributes.items()])
        out.write("\t".join([srna.info_without_attributes,
                             attribute_string]) + "\n")
    for tab in tabs:
        compare_srna_gff(gffs, tab["strain"], tab["strand"], tab["start"],
                         tab["end"], tab["srna_type"], "table")
        tab["info"][15] = tab["srna_type"]
        out_t.write("\t".join(tab["info"]) + "\n")
    os.remove(srna_gff)
    shutil.move(tmp_srna_gff, srna_gff)
    os.remove(srna_table)
    shutil.move(tmp_srna_table, srna_table)



import os
import csv
from Bio import SeqIO
from collections import defaultdict
from annogesiclib.gff3 import Gff3Parser, Gff3Entry
from annogesiclib.TSSpredator import TSSPredatorReader
from annogesiclib.helper import Helper


class Converter(object):
    '''Converting from one format to another format'''

    def __init__(self):
        self.gff3parser = Gff3Parser()
        self.tssparser = TSSPredatorReader()

    def _print_rntptt_file(self, out, entrys, genes):
        '''output to rnt and ptt file'''
        for entry in entrys:
            gene_tag = "-"
            locus_tag = "-"
            location = "..".join([str(entry.start), str(entry.end)])
            length = str(entry.end - entry.start + 1)
            if entry.feature == "CDS":
                if "protein_id" in entry.attributes.keys():
                    pid = entry.attributes["protein_id"]
                else:
                    pid = "-"
                if "locus_tag" in entry.attributes.keys():
                    locus_tag = entry.attributes["locus_tag"]
                elif "Parent" in entry.attributes.keys():
                    for gene in genes:
                        if (gene.attributes["ID"] in 
                                entry.attributes["Parent"].split(",")):
                            if "gene" in gene.attributes.keys():
                                gene_tag = gene.attributes["gene"]
                            locus_tag = gene.attributes["locus_tag"]
                else:
                    locus_tag = "".join([
                        entry.feature, ":", str(entry.start), "-",
                        str(entry.end), "_", entry.strand])
            else:
                pid = "-"
                gene_tag = "-"
                if "locus_tag" in entry.attributes.keys():
                    locus_tag = entry.attributes["locus_tag"]
                elif "Parent" in entry.attributes.keys():
                    for gene in genes:
                        if (gene.attributes["ID"] in 
                                entry.attributes["Parent"].split(",")):
                            if "gene" in gene.attributes.keys():
                                gene_tag = gene.attributes["gene"]
                            locus_tag = gene.attributes["locus_tag"]
                else:
                    locus_tag = "".join([
                        entry.feature, ":", str(entry.start), "-",
                        str(entry.end), "_", entry.strand])
            if "product" in entry.attributes.keys():
                product = entry.attributes["product"]
            else:
                product = "-"
            out.write("\t".join([location, entry.strand, length,
                                 pid, gene_tag, locus_tag, "-", "-",
                                 product]) + "\n")

    def _print_rntptt_title(self, out, num, seq_id, length):
        '''print the title of rnt and ptt file'''
        out.write(seq_id + " - 1.." + length + "\n")
        out.write(num + " proteins\n")
        out.write("\t".join(["Location", "Strand", "Length", "PID",
                  "Gene", "Synonym", "Code", "COG", "Product"]) + "\n")

    def _read_file(self, gff_file, fasta_file, rnas, cdss, genes):
        num_cds = 0
        num_rna = 0
        seq = ""
        g_f = open(gff_file, "r")
        for entry in self.gff3parser.entries(g_f):
            if (entry.feature == "rRNA") or (entry.feature == "tRNA"):
                num_rna += 1
                rnas.append(entry)
            elif entry.feature == "CDS":
                num_cds += 1
                cdss.append(entry)
            elif entry.feature == "gene":
                genes.append(entry)
        g_f.close()
        if fasta_file == "0":
            seq = "-1"
        else:
            with open(fasta_file, "r") as f_f:
                for line in f_f:
                    line = line.strip()
                    if line[0] != ">":
                        seq = seq + line
        return (num_cds, num_rna, seq)

    def _srna2rntptt(self, srna_input_file, srna_output_file, srnas, length):
        '''convert the sRNA gff file to rnt file'''
        num_srna = 0
        r_s = open(srna_input_file, "r")
        for entry in Gff3Parser().entries(r_s):
            num_srna += 1
            srnas.append(entry)
        srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
        r_s.close()
        out_s = open(srna_output_file, "w")
        self._print_rntptt_title(out_s, str(num_srna),
                                 srnas[0].seq_id, str(length))
        num_srna = 0
        for srna in srnas:
            num_srna += 1
            name = '%0*d' % (5, num_srna)
            gene_tag = "-"
            locus_tag = "ncRNA_" + name
            pid = "ncRNA_" + name
            product = "sRNA"
            location = "..".join([str(srna.start), str(srna.end)])
            length = str(srna.end - srna.start + 1)
            out_s.write("\t".join([location, srna.strand, length,
                                   pid, gene_tag, locus_tag, "-", "-",
                                   product]) + "\n")
        out_s.close()

    def _deal_embl_join(self, info):
        '''deal with the embl file which contain join'''
        info = info.replace("(", "")
        info = info.replace(")", "")
        info = info.replace("join", "")
        joins = info.split(",")
        return joins

    def _multi_embl_pos(self, row):
        '''deal with the feature which has multiple positions'''
        poss = []
        if row[21:31] == "complement":
            comple = row[32:-1]
            if comple.find("join") != -1:
                joins = self._deal_embl_join(comple)
                for join in joins:
                    pos = join.split("..")
                    if len(pos) < 2:
                        return "Wrong"
                    poss.append({"start": pos[0], "end": pos[1]})
            else:
                pos = comple.split("..")
                if len(pos) < 2:
                    return "Wrong"
                poss.append({"start": pos[0], "end": pos[1]})
            strand = "-"
        else:
            if row[21:].find("join") != -1:
                joins = self._deal_embl_join(row[21:])
                for join in joins:
                    pos = join.split("..")
                    if len(pos) < 2:
                        return "Wrong"
                    poss.append({"start": pos[0], "end": pos[1]})
            else:
                pos = row[21:].split("..")
                if len(pos) < 2:
                    return "Wrong"
                poss.append({"start": pos[0], "end": pos[1]})
            strand = "+"
        source = row[5:21].rstrip()
        return {"pos": poss, "strand": strand, "source": source}

    def _parser_embl_data(self, embl_file, out):
        '''Parser of embl file for converting to other format'''
        first = True
        line = ""
        note_name = ""
        info = "Wrong"
        with open(embl_file, "r") as f_h:
            for row in f_h:
                row = row.strip()
                if row[0:2] == "SQ":
                    break
                if row[0:2] == "ID":
                    name = row.split(";")
                    name[0] = name[0].replace("ID", "")
                    if "SV" in name[1]:
                        version = name[1].split(" ")[-1]
                        id_name = ".".join([name[0].strip(), version.strip()])
                    else:
                        id_name = name[0].strip()
                if (row[0:2] == "FT"):
                    if row[5] != " ":
                        note_name = row[5:9]
                    if row[5:11] == "source":
                        info = self._multi_embl_pos(row)
                    if (note_name != "misc") and (row[5] == " ") and (
                            row[21] == "/"):
                        if first:
                            first = False
                        else:
                            line = line + ";"
                        data = row[22:].replace(";", ",")
                        data = data.split("=")
                        try:
                            note = data[1].replace("\"", "")
                            line = line + data[0] + "=" + note
                        except:
                            line = line + data[0] + "=" + "True"
                    if (note_name != "misc") and (row[5] == " ") and (
                            row[21] != "/"):
                        note = row[21:].replace("\"", "")
                        note = note.replace(";", ",")
                        line = line + " " + note
                    if (note_name != "misc") and (row[5] != " ") and (
                            row[5:11] != "source"):
                        first = True
                        if info != "Wrong":
                            for pos in info["pos"]:
                                out.write(("{0}\tRefseq\t{1}\t{2}\t{3}"
                                           "\t.\t{4}\t.\t{5}\n").format(
                                          id_name, info["source"],
                                          pos["start"], pos["end"],
                                          info["strand"], line))
                        if (row[5:8] != "CDS") and (row[5:9] != "misc"):
                            info = self._multi_embl_pos(row)
                        elif (row[5:8] == "CDS"):
                            info = self._multi_embl_pos(row)
                        line = ""
        return (id_name, info, line)

    def _assign_tss_type(self, tss, utr_pri, utr_sec):
        '''Assigning the TSS types'''
        if tss.is_primary:
            tss_type = "Primary"
            utr_pri.append(int(tss.utr_length))
        elif tss.is_secondary:
            tss_type = "Secondary"
            utr_sec.append(int(tss.utr_length))
        elif tss.is_internal:
            tss_type = "Internal"
        elif tss.is_antisense:
            tss_type = "Antisense"
        else:
            tss_type = "Orphan"
        return tss_type

    def _multi_tss_class(self, tss, tss_index, tss_features, nums, utrs):
        '''deal with the TSS which has multiple TSS types'''
        tss_type = self._assign_tss_type(tss, utrs["pri"], utrs["sec"])
        if (tss_type not in tss_features["tss_types"]) or (
                tss.locus_tag not in tss_features["locus_tags"]):
            if (tss_type not in tss_features["tss_types"]):
                tss_index[tss_type] += 1
                nums["tss"] += 1
            if (nums["class"] == 1):
                tss_features["tss_types"].append(tss_type)
                tss_features["utr_lengths"].append(
                        tss_type + "_" + tss.utr_length)
                tss_features["locus_tags"].append(tss.locus_tag)
            else:
                if tss_type not in tss_features["tss_types"]:
                    tss_features["tss_types"].append(tss_type)
                    tss_features["utr_lengths"].append(
                            tss_type + "_" + tss.utr_length)
                    tss_features["locus_tags"].append(tss.locus_tag)
            nums["class"] += 1

    def _uni_tss_class(self, tss, utrs, tss_index, tss_features, nums):
        '''It is for TSS which has only one type'''
        tss_type = self._assign_tss_type(tss, utrs["pri"], utrs["sec"])
        tss_index[tss_type] += 1
        tss_features["tss_types"].append(tss_type)
        tss_features["utr_lengths"].append(tss_type+"_"+tss.utr_length)
        tss_features["locus_tags"].append(tss.locus_tag)
        nums["tss"] += 1

    def _print_tssfile(self, nums, tss_features, tss, tss_pro,
                       strain, method, out, tss_libs):
        '''print gff file of TSS'''
        tss_merge_type = ",".join(tss_features["tss_types"])
        utr_length = ",".join(tss_features["utr_lengths"])
        merge_locus_tag = ",".join(tss_features["locus_tags"])
        libs = ",".join(tss_libs)
        strand = Helper().get_strand_name(tss.super_strand)
        attribute_string = ";".join(
                          ["=".join(items) for items in (
                              ["Name", "".join([tss_pro, ":",
                               str(tss.super_pos), "_", strand])],
                              ["ID", strain + "_" + tss_pro.lower() 
                               + str(nums["tss_uni"])],
                              ["type", tss_merge_type],
                              ["utr_length", str(utr_length)],
                              ["associated_gene", merge_locus_tag],
                              ["libs", libs], ["method", "TSSpredator"])])
        out.write("\t".join([strain, method, tss_pro, str(tss.super_pos),
                             str(tss.super_pos), ".", tss.super_strand, ".",
                             attribute_string]) + "\n")

    def convert_gff2rntptt(self, gff_file, fasta_file, ptt_file, rnt_file,
                           srna_input_file, srna_output_file):
        '''Convert gff format to rnt and ptt format'''
        genes = []
        rnas = []
        cdss = []
        srnas = []
        datas = self._read_file(gff_file, fasta_file, rnas, cdss, genes)
        rnas = sorted(rnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
        cdss = sorted(cdss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
        genes = sorted(genes, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
        num_cds = datas[0]
        num_rna = datas[1]
        seq = datas[2]
        out_p = open(ptt_file, "w")
        out_r = open(rnt_file, "w")
        self._print_rntptt_title(out_p, str(num_cds),
                                 cdss[0].seq_id, str(len(seq)))
        self._print_rntptt_file(out_p, cdss, genes)
        if len(rnas) != 0:
            self._print_rntptt_title(out_r, str(num_rna),
                                     rnas[0].seq_id, str(len(seq)))
            self._print_rntptt_file(out_r, rnas, genes)
        out_p.close()
        out_r.close()
        if (srna_input_file is not None) and \
           (srna_output_file is not None):
            self._srna2rntptt(srna_input_file, srna_output_file,
                              srnas, str(len(seq)))
        elif (srna_input_file is None) and \
             (srna_output_file is None):
            pass
        else:
            print("Error: Lack sRNA input gff file or "
                  "the name sRNA output rnt file\n")

    def convert_embl2gff(self, embl_file, gff_file):
        '''Convert embl format to gff format'''
        info = "Wrong"
        out = open(gff_file, "w")
        out.write("##gff-version 3\n")
        datas = self._parser_embl_data(embl_file, out)
        id_name = datas[0]
        info = datas[1]
        line = datas[2]
        if info != "Wrong":
            for pos in info["pos"]:
                out.write(("{0}\tRefseq\t{1}\t{2}\t{3}"
                           "\t.\t{4}\t.\t{5}\n").format(
                          id_name, info["source"], pos["start"],
                          pos["end"], info["strand"], line))
        out.close()

    def _get_libs(self, tss_file):
        '''Get the library which can detect this specific TSS'''
        tss_libs = {}
        tss_fh = open(tss_file, "r")
        for tss in self.tssparser.entries(tss_fh):
            key = "_".join([str(tss.super_pos), tss.super_strand])
            if key not in tss_libs.keys():
                tss_libs[key] = []
            if (tss.is_detected) and (tss.genome not in tss_libs[key]):
                tss_libs[key].append(tss.genome)
        tss_fh.close()
        return tss_libs

    def convert_mastertable2gff(self, tss_file, method, tss_pro,
                                strain, out_gff):
        '''Convert MasterTable to gff format'''
        temps = {"tss": 0, "strand": "#"}
        nums = {"tss": 0, "tss_uni": 0, "class": 1}
        check_print = False
        utrs = {"total": [], "pri": [], "sec": []}
        tss_features = {"tss_types": [], "locus_tags": [], "utr_lengths": []}
        tss_index = defaultdict(lambda: 0)
        tss_fh = open(tss_file, "r")
        out = open(out_gff, "w")
        out.write("##gff-version 3\n")
        tss_libs = self._get_libs(tss_file)
        detect_run = False
        for tss in self.tssparser.entries(tss_fh):
            detect_run = True
            key = "_".join([str(tss.super_pos), tss.super_strand])
            if ((tss.super_pos == temps["tss"])) and (
                    temps["strand"] == tss.super_strand) and (
                        tss.class_count == 1):
                pass
            else:
                if ((tss.super_pos != temps["tss"])) or (
                        temps["strand"] != tss.super_strand):
                    check_print = False
                    nums["class"] = 1
                    if tss.utr_length != "NA":
                        utrs["total"].append(int(tss.utr_length))
                temps["tss"] = tss.super_pos
                temps["strand"] = tss.super_strand
                if (tss.class_count != 1) and (
                        nums["class"] <= tss.class_count):
                    self._multi_tss_class(tss, tss_index, tss_features,
                                          nums, utrs)
                if (tss.class_count == 1) or (
                        nums["class"] > tss.class_count):
                    if (tss.class_count == 1):
                        self._uni_tss_class(tss, utrs, tss_index,
                                            tss_features, nums)
                    if (check_print is False):
                        self._print_tssfile(nums, tss_features, tss, tss_pro,
                                            strain, method, out, tss_libs[key])
                        check_print = True
                        nums["tss_uni"] += 1
                    tss_features = {"tss_types": [], "locus_tags": [],
                                    "utr_lengths": []}
        if (check_print is False) and detect_run:
            self._print_tssfile(nums, tss_features, tss, tss_pro,
                                strain, method, out, tss_libs[key])
        tss_fh.close()
        out.close()

    def convert_transtermhp2gff(self, transterm_file, gff_file):
        '''Convert the output of TransTermHP to gff format'''
        out = open(gff_file, "w")
        out.write("##gff-version 3\n")
        terms = []
        with open(transterm_file) as t_h:
            for line in t_h:
                row = line[:-1].split()
                if len(row) < 10:
                    continue
                if len(row) == 14:
                    start = row[1]
                    end = row[3]
                    strand = row[4]
                    gene = row[0]
                else:
                    start = row[0]
                    end = row[2]
                    strand = row[3]
                    gene = "missing"
                entry = Gff3Entry({
                    "seq_id": transterm_file.split("/")[-1][:-1 * len(
                               "_best_terminator_after_gene.bag")],
                    "source": "TransTermHP",
                    "feature": "terminator",
                    "start": start,
                    "end": end,
                    "score": ".",
                    "strand": strand,
                    "phase": ".",
                    "attributes": "associated_gene=%s" % (gene)
                    })
                terms.append(entry)
        sort_terms = sorted(terms, key=lambda k: (k.seq_id, k.start,
                                                  k.end, k.strand))
        num = 0
        for term in sort_terms:
            out.write("\t".join([str(field) for field in [
                                term.seq_id, term.source, term.feature,
                                term.start, term.end, term.score,
                                term.strand, term.phase,
                                term.attribute_string]]))
            name = '%0*d' % (5, num)
            out.write(";ID={0}_terminator{1};Name=terminator_{2}\n".format(
                      term.seq_id, num, name))
            num += 1
        out.close()

    def convert_circ2gff(self, circ_file, args_circ, out_all, out_filter):
        '''Convert the circRNA output of segemehl to gff format'''
        circs = []
        out_a = open(out_all, "w")
        out_f = open(out_filter, "w")
        out_a.write("##gff-version 3\n")
        out_f.write("##gff-version 3\n")
        f_h = open(circ_file, "r")
        for row in csv.reader(f_h, delimiter='\t'):
            if row[0] != "ID":
                circs.append({"name": row[0], "strain": row[1],
                              "strand": row[2], "start": int(row[3]),
                              "end": int(row[4]), "conflict": row[5],
                              "depth": int(row[6]), "per_start": float(row[7]),
                              "per_end": float(row[8])})
        circs = sorted(circs, key=lambda k: (k["strain"], k["start"],
                                             k["end"], k["strand"]))
        for circ in circs:
            id_ = circ["name"].split("_")[1]
            attribute_string = ";".join(["=".join(items) for items in [
                                           ("ID", circ["strain"] +
                                            "_circrna" + id_),
                                           ("name", circ["name"]),
                                           ("support_reads",
                                            str(circ["depth"])),
                                           ("read_at_start",
                                            str(circ["per_start"])),
                                           ("read_at_end",
                                            str(circ["per_end"])),
                                           ("conflict", circ["conflict"]),
                                           ("method", "segemehl")]])
            out_a.write("\t".join([str(field) for field in [
                            circ["strain"], "ANNOgesic", "circRNA",
                            str(circ["start"]), str(circ["end"]),
                            ".", circ["strand"], ".",
                            attribute_string]]) + "\n")
            if (circ["depth"] >= args_circ.support) and (
                    circ["conflict"] == "NA") and (
                    circ["per_start"] >= args_circ.start_ratio) and (
                    circ["per_end"] >= args_circ.end_ratio):
                out_f.write("\t".join([str(field) for field in [
                            circ["strain"], "ANNOgesic", "circRNA",
                            str(circ["start"]), str(circ["end"]),
                            ".", circ["strand"], ".",
                            attribute_string]]) + "\n")
        f_h.close()
        out_a.close()
        out_f.close()

    def convert_gbk2embl(self, input_folder):
        """Convert gbk to embl."""
        print("Converting gbk to embl")
        for annotation_file in os.listdir(input_folder):
            if annotation_file[-3:] == "gbk":
                gbk_file = annotation_file
                embl_file = gbk_file[0:-3] + "embl"
                gbk_entry = SeqIO.parse(os.path.join(
                                        input_folder, gbk_file), "genbank")
                count = SeqIO.write(gbk_entry, os.path.join(
                                    input_folder, embl_file), "embl")
                print("Converted %i records" % count)



import os
import sys
import shutil
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.format_fixer import FormatFixer
from annogesiclib.extract_psortb import extract_psortb
from annogesiclib.stat_sublocal import stat_sublocal
from annogesiclib.gff3 import Gff3Parser


class SubLocal(object):
    '''detection of subcellular localization'''

    def __init__(self, args_sub):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.fixer = FormatFixer()
        self.gff_path = os.path.join(args_sub.gffs, "tmp")
        self.fasta_path = os.path.join(args_sub.fastas, "tmp")
        if args_sub.trans is not None:
            self.tran_path = os.path.join(args_sub.trans, "tmp")
        else:
            self.tran_path = None
        self.out_all = os.path.join(args_sub.out_folder, "all_CDS")
        self.out_express = os.path.join(args_sub.out_folder, "expressed_CDS")
        self.all_tmp_path = os.path.join(self.out_all, "tmp")
        self.express_tmp_path = os.path.join(self.out_express, "tmp")
        self.all_stat_path = os.path.join(self.out_all, "statistics")
        self.express_stat_path = os.path.join(self.out_express, "statistics")
        self.all_tmp_result = os.path.join(self.out_all, "tmp_results")
        self.express_tmp_result = os.path.join(self.out_express, "tmp_results")
        self.all_result = os.path.join(self.out_all, "psortb_results")
        self.express_result = os.path.join(self.out_express, "psortb_results")
        self.endfix_table = "table.csv"
        self.endfix_raw = "raw.txt"
        self._make_folder()

    def _make_folder(self):
        self.helper.check_make_folder(self.out_all)
        self.helper.check_make_folder(self.out_express)
        self.helper.check_make_folder(self.all_stat_path)
        self.helper.check_make_folder(self.express_stat_path)
        self.helper.check_make_folder(self.all_result)
        self.helper.check_make_folder(self.express_result)

    def _compare_cds_tran(self, gff_file, tran_file):
        '''compare CDS and transcript to find the expressed CDS'''
        out = open(os.path.join(self.out_all, "tmp_cds.gff"), "w")
        cdss = []
        fh = open(gff_file)
        th = open(tran_file)
        for entry in Gff3Parser().entries(fh):
            if entry.feature == "CDS":
                cdss.append(entry)
        trans = []
        for entry in Gff3Parser().entries(th):
            trans.append(entry)
        for cds in cdss:
            for ta in trans:
                if (cds.strand == ta.strand) and (
                        cds.seq_id == ta.seq_id):
                    if ((cds.end < ta.end) and (
                             cds.end > ta.start) and (
                             cds.start <= ta.start)) or (
                            (cds.start > ta.start) and (
                             cds.start < ta.end) and (
                             cds.end >= ta.end)) or (
                            (cds.end >= ta.end) and (
                             cds.start <= ta.start)) or (
                            (cds.end <= ta.end) and (
                             cds.start >= ta.start)):
                        out.write(cds.info + "\n")
                        break
        fh.close()
        th.close()
        out.close()

    def _get_protein_seq(self, gff, tmp_path, tran_path):
        prefix = gff.replace(".gff", "")
        fasta = self.helper.get_correct_file(self.fasta_path, ".fa",
                                             prefix, None, None)
        dna_seq_file = os.path.join(tmp_path, "_".join([prefix, "dna.fa"]))
        print("Generating CDS fasta files of {0}".format(prefix))
        if tran_path is not None:
            self._compare_cds_tran(os.path.join(self.gff_path, gff),
                                   os.path.join(tran_path, "_".join([
                                       prefix, "transcript.gff"])))
            self.helper.get_cds_seq(os.path.join(self.out_all, "tmp_cds.gff"),
                                    fasta, dna_seq_file)
            os.remove(os.path.join(self.out_all, "tmp_cds.gff"))
        else:
            self.helper.get_cds_seq(os.path.join(self.gff_path, gff),
                                    fasta, dna_seq_file)
        print("Transfering DNA seq to protein seq of {0}".format(prefix))
        self.helper.translation(dna_seq_file, "tmp")
        prot_seq_file = os.path.join(
                tmp_path, "_".join([prefix, "protein.fa"]))
        self.fixer.fix_emboss("tmp", prot_seq_file)
        os.remove("tmp")
        return prefix

    def _psortb(self, psortb_path, strain_type, prot_seq_file,
                out_raw, out_err):
        call([psortb_path, strain_type, prot_seq_file],
             stdout=out_raw, stderr=out_err)

    def _run_psortb(self, args_sub, prefix, out_folder, tmp_path, tmp_result):
        print("Running psortb of {0}".format(prefix))
        out_err = open(os.path.join(out_folder, "tmp_log"), "w")
        out_raw = open(os.path.join(tmp_result,
                       "_".join([prefix, self.endfix_raw])), "w")
        prot_seq_file = os.path.join(tmp_path,
                                     "_".join([prefix, "protein.fa"]))
        if args_sub.gram == "positive":
            self._psortb(args_sub.psortb_path, "-p", prot_seq_file,
                         out_raw, out_err)
        elif args_sub.gram == "negative":
            self._psortb(args_sub.psortb_path, "-n", prot_seq_file,
                         out_raw, out_err)
        else:
            print("Error: It is not a proper bacteria type - {0}!!".format(
                  args_sub.gram))
            sys.exit()
        out_err.close()
        out_raw.close()

    def _extract_result(self, args_sub, tmp_psortb_path, prefix, gff_file):
        '''extract the result of psortb'''
        if args_sub.merge:
            print("Merging gff")
            extract_psortb(os.path.join(
                tmp_psortb_path, "_".join([prefix, self.endfix_raw])),
                os.path.join(tmp_psortb_path, "_".join([
                    prefix, self.endfix_table])),
                gff_file, os.path.join(prefix + ".gff"),
                args_sub.fuzzy)
            shutil.move(prefix + ".gff", gff_file)
        else:
            extract_psortb(os.path.join(
                tmp_psortb_path, "_".join([prefix, self.endfix_raw])),
                os.path.join(tmp_psortb_path, "_".join([
                    prefix, self.endfix_table])),
                None, None, args_sub.fuzzy)

    def _merge_and_stat(self, gffs, tmp_psortb_path, stat_path, psortb_result):
        for folder in os.listdir(gffs):
            if folder.endswith(".gff_folder"):
                prefix = folder.replace(".gff_folder", "")
                self.helper.check_make_folder(
                     os.path.join(psortb_result, prefix))
                merge_table = os.path.join(
                        psortb_result, prefix,
                        "_".join([prefix, self.endfix_table]))
                for gff in os.listdir(os.path.join(gffs, folder)):
                    result = self.helper.get_correct_file(
                            tmp_psortb_path, "_" + self.endfix_raw,
                            gff.replace(".gff", ""), None, None)
                    shutil.copy(result, os.path.join(psortb_result, prefix))
                    result = self.helper.get_correct_file(
                            tmp_psortb_path, "_" + self.endfix_table,
                            gff.replace(".gff", ""), None, None)
                    self.helper.merge_file(result, merge_table)
                self.helper.check_make_folder(os.path.join(stat_path, prefix))
                stat_sublocal(merge_table,
                              os.path.join(
                                  stat_path, prefix, prefix),
                              os.path.join(
                                  stat_path, prefix, "_".join([
                                      "stat", prefix, "sublocal.csv"])))

    def _remove_tmps(self, args_sub):
        self.helper.remove_tmp_dir(args_sub.fastas)
        self.helper.remove_tmp_dir(args_sub.gffs)
        self.helper.remove_all_content(args_sub.out_folder, "tmp", "dir")
        self.helper.remove_all_content(self.out_all, "tmp", "dir")
        self.helper.remove_all_content(self.out_express, "tmp", "dir")
        os.remove(os.path.join(self.out_all, "tmp_log"))
        if args_sub.trans is not None:
            os.remove(os.path.join(self.out_express, "tmp_log"))
            self.helper.remove_tmp_dir(args_sub.trans)

    def run_sub_local(self, args_sub):
        for gff in os.listdir(args_sub.gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(
                                                 args_sub.gffs, gff))
        self.multiparser.parser_gff(args_sub.gffs, None)
        self.multiparser.parser_fasta(args_sub.fastas)
        if args_sub.trans is not None:
            self.multiparser.parser_gff(args_sub.trans, "transcript")
            self.helper.check_make_folder(self.express_tmp_path)
            self.helper.check_make_folder(self.express_tmp_result)
        self.helper.check_make_folder(self.all_tmp_path)
        self.helper.check_make_folder(self.all_tmp_result)
        for gff in os.listdir(self.gff_path):
            if args_sub.trans is not None:
                print("Running expressed gene now")
                prefix = self._get_protein_seq(gff, self.express_tmp_path,
                                               self.tran_path)
                self._run_psortb(args_sub, prefix, self.out_express,
                                 self.express_tmp_path,
                                 self.express_tmp_result)
                self._extract_result(args_sub, self.express_tmp_result, prefix,
                                     os.path.join(self.gff_path, gff))
            print("Running all gene now")
            prefix = self._get_protein_seq(gff, self.all_tmp_path, None)
            self._run_psortb(args_sub, prefix, self.out_all,
                             self.all_tmp_path, self.all_tmp_result)
            self._extract_result(args_sub, self.all_tmp_result, prefix,
                                 os.path.join(self.gff_path, gff))
        self._merge_and_stat(args_sub.gffs, self.all_tmp_result,
                             self.all_stat_path, self.all_result)
        if args_sub.trans is not None:
            self._merge_and_stat(args_sub.gffs, self.express_tmp_result,
                                 self.express_stat_path, self.express_result)
        self._remove_tmps(args_sub)



import os
import shutil
import time
from subprocess import Popen
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.potential_target import potential_target
from annogesiclib.format_fixer import FormatFixer
from annogesiclib.merge_rnaplex_rnaup import merge_srna_target
from annogesiclib.gff3 import Gff3Parser


class sRNATargetPrediction(object):
    '''detection of sRNA-target interaction'''

    def __init__(self, args_tar):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.fixer = FormatFixer()
        self.gff_parser = Gff3Parser()
        self.target_seq_path = os.path.join(args_tar.out_folder, "target_seqs")
        self.srna_seq_path = os.path.join(args_tar.out_folder, "sRNA_seqs")
        self.rnaplex_path = os.path.join(args_tar.out_folder, "RNAplex")
        self.rnaup_path = os.path.join(args_tar.out_folder, "RNAup")
        self.merge_path = os.path.join(args_tar.out_folder, "merge")
        self.srna_path = os.path.join(args_tar.srnas, "tmp")
        self.fasta_path = os.path.join(args_tar.fastas, "tmp")
        self.gff_path = os.path.join(args_tar.gffs, "tmp")
        self.tmps = {"tmp": "tmp", "rnaup": "tmp_rnaup", "log": "tmp_log",
                     "all_fa": "tmp*.fa", "all_txt": "tmp*.txt"}

    def _check_gff(self, gffs):
        for gff in os.listdir(gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(gffs, gff))

    def _run_rnaplfold(self, rnaplfold_path, file_type, win_size, span,
                       unstr_region, seq_path, prefix, out_path):
        current = os.getcwd()
        os.chdir(out_path)
        command = " ".join([rnaplfold_path,
                            "-W", str(win_size),
                            "-L", str(span),
                            "-u", str(unstr_region),
                            "-O"])
        if file_type == "sRNA":
            os.system("<".join([command, os.path.join(current, seq_path,
                                "_".join([self.tmps["tmp"], prefix,
                                          file_type + ".fa"]))]))
        else:
            os.system("<".join([command, os.path.join(current, seq_path,
                                "_".join([prefix, file_type + ".fa"]))]))
        os.chdir(current)

    def _wait_process(self, processes):
        for p in processes:
            p.wait()
            if p.stdout:
                p.stdout.close()
            if p.stdin:
                p.stdin.close()
            if p.stderr:
                p.stderr.close()
            try:
                p.kill()
            except OSError:
                pass
            time.sleep(5)

    def _sort_srna_fasta(self, fasta, prefix, path):
        out = open(os.path.join(path,
                   "_".join([self.tmps["tmp"], prefix, "sRNA.fa"])), "w")
        srnas = []
        with open(fasta) as f_h:
            for line in f_h:
                line = line.strip()
                if line.startswith(">"):
                    name = line[1:]
                else:
                    srnas.append({"name": name, "seq": line, "len": len(line)})
        srnas = sorted(srnas, key=lambda x: (x["len"]))
        for srna in srnas:
            out.write(">" + srna["name"].split("|")[0] + "\n")
            out.write(srna["seq"] + "\n")
        out.close()

    def _read_fasta(self, fasta_file):
        seq = ""
        with open(fasta_file, "r") as seq_f:
            for line in seq_f:
                line = line.strip()
                if line.startswith(">"):
                    continue
                else:
                    seq = seq + line
        return seq

    def _get_specific_seq(self, srna_file, seq_file, srna_out, querys):
        for query in querys:
            srna_datas = query.split(":")
            srna = {"seq_id": srna_datas[0], "strand": srna_datas[3],
                    "start": int(srna_datas[1]), "end": int(srna_datas[2])}
            gff_f = open(srna_file, "r")
            out = open(srna_out, "a")
            seq = self._read_fasta(seq_file)
            num = 0
            for entry in self.gff_parser.entries(gff_f):
                if (entry.seq_id == srna["seq_id"]) and (
                        entry.strand == srna["strand"]) and (
                        entry.start == srna["start"]) and (
                        entry.end == srna["end"]):
                    if "ID" in entry.attributes.keys():
                        id_ = entry.attributes["ID"]
                    else:
                        id_ = entry.feature + str(num)
                    gene = self.helper.extract_gene(seq, entry.start,
                                                    entry.end, entry.strand)
                    out.write(">{0}|{1}|{2}|{3}|{4}\n{5}\n".format(
                              id_, entry.seq_id, entry.start,
                              entry.end, entry.strand, gene))
                    num += 1
            gff_f.close()
            out.close()

    def _gen_seq(self, prefixs, args_tar):
        print("Generating sRNA fasta files")
        for srna in os.listdir(self.srna_path):
            if srna.endswith("_sRNA.gff"):
                prefix = srna.replace("_sRNA.gff", "")
                prefixs.append(prefix)
                srna_out = os.path.join(self.srna_seq_path,
                                        "_".join([prefix, "sRNA.fa"]))
                if "all" in args_tar.query:
                    self.helper.get_seq(
                            os.path.join(self.srna_path, srna),
                            os.path.join(self.fasta_path, prefix + ".fa"),
                            srna_out)
                else:
                    if "_".join([prefix, "sRNA.fa"]) in os.listdir(
                       self.srna_seq_path):
                        os.remove(srna_out)
                    self._get_specific_seq(
                            os.path.join(self.srna_path, srna),
                            os.path.join(self.fasta_path, prefix + ".fa"),
                            srna_out, args_tar.query)
                self._sort_srna_fasta(srna_out, prefix, self.srna_seq_path)
        print("Generating target fasta files")
        for gff in os.listdir(self.gff_path):
            if gff.endswith(".gff"):
                prefix = gff.replace(".gff", "")
                potential_target(os.path.join(self.gff_path, gff),
                                 os.path.join(self.fasta_path, prefix + ".fa"),
                                 os.path.join(self.target_seq_path), args_tar)
                file_num = 1
                num = 0
                sub_prefix = os.path.join(self.target_seq_path,
                                          "_".join([prefix, "target"]))
                sub_out = open("_".join([sub_prefix, str(file_num) + ".fa"]),
                               "w")
                with open((sub_prefix + ".fa"), "r") as t_f:
                    for line in t_f:
                        line = line.strip()
                        if line.startswith(">"):
                            num += 1
                        if (num == 100):
                            num = 0
                            file_num += 1
                            sub_out.close()
                            sub_out = open("_".join([sub_prefix,
                                           str(file_num) + ".fa"]), "w")
                        sub_out.write(line + "\n")
                sub_out.close()

    def _run_rnaplex(self, prefix, rnaplfold_folder, args_tar):
        print("Running RNAplex of {0}".format(prefix))
        num_process = 0
        processes = []
        for seq in os.listdir(self.target_seq_path):
            if (prefix in seq) and ("_target_" in seq):
                print("Running RNAplex with {0}".format(seq))
                out_rnaplex = open(os.path.join(
                    self.rnaplex_path, prefix, "_".join([
                        prefix, "RNAplex", str(num_process) + ".txt"])), "w")
                num_process += 1
                p = Popen([args_tar.rnaplex_path,
                           "-q", os.path.join(
                               self.srna_seq_path, "_".join([
                                   self.tmps["tmp"], prefix, "sRNA.fa"])),
                           "-t", os.path.join(self.target_seq_path, seq),
                           "-l", str(args_tar.inter_length),
                           "-e", str(args_tar.energy),
                           "-z", str(args_tar.duplex_dist),
                           "-a", rnaplfold_folder], stdout=out_rnaplex)
                processes.append(p)
                if num_process % args_tar.core_plex == 0:
                    self._wait_process(processes)
        self._wait_process(processes)
        return num_process

    def _rna_plex(self, prefixs, args_tar):
        for prefix in prefixs:
            print("Running RNAplfold of {0}".format(prefix))
            self.helper.check_make_folder(
                        os.path.join(self.rnaplex_path, prefix))
            rnaplfold_folder = os.path.join(self.rnaplex_path, prefix,
                                          "RNAplfold")
            os.mkdir(rnaplfold_folder)
            self._run_rnaplfold(
                args_tar.rnaplfold_path, "sRNA", args_tar.win_size_s,
                args_tar.span_s, args_tar.unstr_region_rnaplex_s,
                self.srna_seq_path, prefix, rnaplfold_folder)
            self._run_rnaplfold(
                args_tar.rnaplfold_path, "target", args_tar.win_size_t,
                args_tar.span_t, args_tar.unstr_region_rnaplex_t,
                self.target_seq_path, prefix, rnaplfold_folder)
            num_process = self._run_rnaplex(prefix, rnaplfold_folder, args_tar)
            rnaplex_file = os.path.join(self.rnaplex_path, prefix,
                                        "_".join([prefix, "RNAplex.txt"]))
            if ("_".join([prefix, "RNAplex.txt"]) in
                    os.listdir(os.path.join(self.rnaplex_path, prefix))):
                os.remove(rnaplex_file)
            for index in range(0, num_process):
                self.helper.merge_file(os.path.join(
                    self.rnaplex_path, prefix, "_".join([
                        prefix, "RNAplex", str(index) + ".txt"])),
                    rnaplex_file)
            self.helper.remove_all_content(os.path.join(
                 self.rnaplex_path, prefix), "_RNAplex_", "file")
            self.fixer.fix_rnaplex(rnaplex_file, self.tmps["tmp"])
            shutil.move(self.tmps["tmp"], rnaplex_file)

    def _run_rnaup(self, num_up, processes, out_rnaup, out_log, args_tar):
        for index in range(1, num_up + 1):
            out_tmp_up = open(os.path.join(
                args_tar.out_folder, "".join([self.tmps["rnaup"],
                                              str(index), ".txt"])), "w")
            out_err = open(os.path.join(
                args_tar.out_folder, "".join([self.tmps["log"],
                                              str(index), ".txt"])), "w")
            in_up = open(os.path.join(
                args_tar.out_folder, "".join([self.tmps["tmp"],
                                              str(index), ".fa"])), "r")
            p = Popen([args_tar.rnaup_path,
                       "-u", str(args_tar.unstr_region_rnaup),
                       "-o", "--interaction_first"],
                      stdin=in_up, stdout=out_tmp_up, stderr=out_err)
            processes.append(p)
        if len(processes) != 0:
            time.sleep(5)
            self._wait_process(processes)
            os.system("rm " + os.path.join(args_tar.out_folder,
                                           self.tmps["all_fa"]))
            self._merge_txt(num_up, out_rnaup, out_log, args_tar.out_folder)
            os.system("rm " + os.path.join(args_tar.out_folder,
                                           self.tmps["all_txt"]))

    def _merge_txt(self, num_up, out_rnaup, out_log, out_folder):
        for index in range(1, num_up + 1):
            self.helper.merge_file(
                os.path.join(out_folder, "".join([self.tmps["rnaup"],
                                                  str(index), ".txt"])),
                out_rnaup)
            self.helper.merge_file(
                os.path.join(out_folder, "".join([self.tmps["log"],
                                                  str(index), ".txt"])),
                out_log)

    def _get_continue(self, out_rnaup):
        '''For RNAup, it can continue running RNAup based on previous run'''
        srnas = []
        matchs = {}
        out = open("tmp.txt", "w")
        with open(out_rnaup) as f_h:
            for line in f_h:
                line = line.strip()
                if ">srna" in line:
                    srna = line[1:]
                    srnas.append(srna)
                    matchs[srna] = []
                else:
                    matchs[srna].append(line)
        srnas = srnas[:-1]
        for srna in srnas:
            out.write(">" + srna + "\n")
            for target in matchs[srna]:
                out.write(target + "\n")
        out.close()
        os.remove(out_rnaup)
        shutil.move("tmp.txt", out_rnaup)
        return srnas

    def _rnaup(self, prefixs, args_tar):
        for prefix in prefixs:
            srnas = []
            print("Running RNAup of {0}".format(prefix))
            if not os.path.exists(os.path.join(self.rnaup_path, prefix)):
                os.mkdir(os.path.join(self.rnaup_path, prefix))
            num_up = 0
            processes = []
            out_rnaup = os.path.join(self.rnaup_path, prefix,
                                     "_".join([prefix + "_RNAup.txt"]))
            out_log = os.path.join(self.rnaup_path, prefix,
                                   "_".join([prefix + "_RNAup.log"]))
            if "_".join([prefix, "RNAup.txt"]) in \
                    os.listdir(os.path.join(self.rnaup_path, prefix)):
                if not args_tar.continue_rnaup:
                    os.remove(out_rnaup)
                    os.remove(out_log)
                else:
                    srnas = self._get_continue(out_rnaup)
            with open(os.path.join(self.srna_seq_path, "_".join([
                    self.tmps["tmp"], prefix, "sRNA.fa"])), "r") as s_f:
                for line in s_f:
                    line = line.strip()
                    if line.startswith(">"):
                        if line[1:] in srnas:
                            start = False
                            continue
                        start = True
                        print("Running RNAup with {0}".format(line[1:]))
                        num_up += 1
                        out_up = open(os.path.join(args_tar.out_folder,
                                      "".join([self.tmps["tmp"],
                                               str(num_up), ".fa"])), "w")
                        out_up.write(line + "\n")
                    else:
                        if start:
                            out_up.write(line + "\n")
                            out_up.close()
                            self.helper.merge_file(os.path.join(
                                self.target_seq_path,
                                "_".join([prefix, "target.fa"])),
                                os.path.join(args_tar.out_folder,
                                             "".join([self.tmps["tmp"],
                                                      str(num_up), ".fa"])))
                            if num_up == args_tar.core_up:
                                self._run_rnaup(num_up, processes,
                                                out_rnaup, out_log, args_tar)
                                processes = []
                                num_up = 0
            self._run_rnaup(num_up, processes, out_rnaup, out_log, args_tar)

    def _merge_rnaplex_rnaup(self, prefixs, args_tar):
        '''merge the result of RNAup and RNAplex'''
        for prefix in prefixs:
            rnaplex_file = None
            rnaup_file = None
            out_rnaplex = None
            out_rnaup = None
            self.helper.check_make_folder(os.path.join(
                                          self.merge_path, prefix))
            print("Ranking {0} now".format(prefix))
            if (args_tar.program == "both") or (args_tar.program == "RNAplex"):
                rnaplex_file = os.path.join(self.rnaplex_path, prefix,
                                            "_".join([prefix, "RNAplex.txt"]))
                out_rnaplex = os.path.join(
                        self.rnaplex_path, prefix,
                        "_".join([prefix, "RNAplex_rank.csv"]))
            if (args_tar.program == "both") or (args_tar.program == "RNAup"):
                rnaup_file = os.path.join(self.rnaup_path, prefix,
                                          "_".join([prefix, "RNAup.txt"]))
                out_rnaup = os.path.join(self.rnaup_path, prefix,
                                         "_".join([prefix, "RNAup_rank.csv"]))
            merge_srna_target(rnaplex_file, rnaup_file, args_tar,
                              out_rnaplex, out_rnaup,
                              os.path.join(self.fasta_path, prefix + ".fa"),
                              os.path.join(self.merge_path, prefix,
                                           "_".join([prefix, "merge.csv"])),
                              os.path.join(self.merge_path, prefix,
                                           "_".join([prefix, "overlap.csv"])),
                              os.path.join(self.srna_path,
                                           "_".join([prefix, "sRNA.gff"])),
                              os.path.join(self.gff_path, prefix + ".gff"))

    def run_srna_target_prediction(self, args_tar):
        self._check_gff(args_tar.gffs)
        self._check_gff(args_tar.srnas)
        self.multiparser.parser_gff(args_tar.gffs, None)
        self.multiparser.parser_fasta(args_tar.fastas)
        self.multiparser.parser_gff(args_tar.srnas, "sRNA")
        prefixs = []
        self._gen_seq(prefixs, args_tar)
        if (args_tar.program == "both") or (
                args_tar.program == "RNAplex"):
            self._rna_plex(prefixs, args_tar)
        self.helper.remove_all_content(self.target_seq_path,
                                       "_target_", "file")
        if (args_tar.program == "both") or (
                args_tar.program == "RNAup"):
            self._rnaup(prefixs, args_tar)
        self._merge_rnaplex_rnaup(prefixs, args_tar)
        if (args_tar.program == "RNAplex") or (
                args_tar.program == "both"):
            for strain in os.listdir(os.path.join(
                          args_tar.out_folder, "RNAplex")):
                shutil.rmtree(os.path.join(args_tar.out_folder, "RNAplex",
                                           strain, "RNAplfold"))
        self.helper.remove_all_content(args_tar.out_folder,
                                       self.tmps["tmp"], "dir")
        self.helper.remove_all_content(args_tar.out_folder,
                                       self.tmps["tmp"], "file")
        self.helper.remove_tmp_dir(args_tar.gffs)
        self.helper.remove_tmp_dir(args_tar.srnas)
        self.helper.remove_tmp_dir(args_tar.fastas)
        self.helper.remove_all_content(self.srna_seq_path, "tmp_", "file")
